<?php

namespace OrganizationsManagement\Service;

use Doctrine\ORM\EntityManager;
use OrganizationsManagement\Util\InstanceProvider;
use OrganizationsManagement\Factory;
use OrganizationsManagement\Service;
use OrganizationsManagement\Util\Messages;
use OrganizationsManagement\Util\Utilities;
use OrganizationsManagement\Util\OpensslCrypt;
use OrganizationsManagement\Util\DBConnection;
use Teknisa\Model\Geral\Entities as GeralEntities;
use Setforms2\Shared\ORM\DateTime as SetformsDateTime;
use Teknisa\Model\Geral\Repositories as GeralRepositories;
use Teknisa\Model\Teknisa\Repositories as TeknisaRepositories;
use Teknisa\Model\Teknisa\Entities as TeknisaEntities;
use Teknisa\Model\Teknisa\Entities\OmProduto;

class Organizacao
{
    const TIPO_CLIENTE_PARCEIRO       = 'CLIENTE';
    const USE_IP_RESTRICTION_NEGATIVE = 'N';
    const ALLOW_PRODUCT_CALL_NEGATIVE = 'N';

    const IS_ORGANIZATION_SAAS = 'S';

    /** @var EntityManager $entityManager */
    private $entityManager;

    /** @var GeralRepositories\Organizacao */
    private $repositoryOrganizacao;

    /** @var GeralRepositories\Cliente */
    private $repositoryCliente;

    /** @var Factory\Organizacao */
    private $factoryOrganizacao;

    /** @var GeralRepositories\Empresa  */
    private $repositoryEmpresa;

    /** @var GeralRepositories\Inscrestad */
    private $repositoryInscricaoEstadual;

    /** TeknisaRepositories\OmIpRestriction */
    private $repositoryIpRestriction;

    /** @var Factory\InscricaoEstadual  */
    private $factoryInscricaoEstadual;

    /** @var Factory\InscricaoMunicipal  */
    private $factoryInscricaoMunicipal;

    /** @var Factory\Filial  */
    private $factoryFilial;

    /** @var Factory\EnderecoFilial  */
    private $factoryEnderecoFilial;

    /** @var GeralRepositories\Parceria */
    private $repositoryParceria;

    /** @var Factory\ParceiroNegocio  */
    private $factoryParceiroNegocio;

    /** @var Factory\Parceria  */
    private $factoryParceria;

    /** @var GeralRepositories\Operador  */
    private $repositoryOperador;

    /** @var Factory\Operador  */
    private $factoryOperador;

    /** @var Factory\PerfilAcesso  */
    private $factoryPerfilAcesso;

    /** @var Factory\PerfilAcessoParceiro  */
    private $factoryPerfilAcessoParceiro;

    /** @var Factory\ParametroGeral  */
    private $factoryParametroGeral;

    /** @var Factory\OrganizacaoOperador  */
    private $factoryOrganizacaoOperador;

    /** @var Factory\OrganizationParameters  */
    private $factoryOrganizationParameters;

    /** @var Factory\Cliente  */
    private $factoryCliente;

    /** @var Factory\EndeClie  */
    private $factoryEndeClie;

    /** @var Factory\EnderecoParc  */
    private $factoryEnderecoParc;

    /** @var Operador $serviceOperador */
    private $serviceOperador;

    /** @var  */
    private $senhaGeradaParaOperador;
    private $emailOperadorCriado;
    private $codigoOperadorCriado;
    private $nomeOperadorCriado;

    /** @var \Teknisa\Libs\Util\Environment  */
    private $environment;

    /** @var Email\EmailSolicitacao  */
    private $serviceEmailSolicitacao;

    /** @var Factory\Estruturam */
    private $factoryEstruturam;

    /** @var Factory\Hierarquiaorg */
    private $factoryHierarquiaorg;

    /** @var GeralRepositories\Estruturaoper */
    private $repositoryEstruturaoper;

    /** @var TeknisaRepositories\OmProduto */
    private $repositoryOmProduto;

    /** @var TeknisaRepositories\OmProdutoOrg */
    private $repositoryOmProdutoOrg;

    /** @var TeknisaRepositories\OmProdutoOrgElemento */
    private $repositoryOmProdutoOrgElemento;

    /** @var TeknisaRepositories\OmProdutoElemento */
    private $repositoryOmProdutoElemento;

    /** @var TeknisaRepositories\OmParamOrg */
    private $repositoryParamOrg;

    /** @var GeralRepositories\Organizacaooper */
    private $repositoryOrganizacaoOperador;

    /** @var Factory\Estruturaoper */
    private $factoryEstruturaoper;

    /** @var Factory\IpRestriction */
    private $factoryIpRestriction;

    /** @var Factory\Empresa */
    private $factoryCompany;

    /** @var DBConnection */
    private $dbConnection;

    /** @var Utilities */
    private $utilities;

    public function __construct(
        EntityManager $entityManager,
        Factory\Organizacao $organizationFactory,
        Factory\Empresa $factoryCompany,
        Factory\InscricaoEstadual $stateInscriptionFactory,
        Factory\InscricaoMunicipal $factoryInscricaoMunicipal,
        Factory\Filial $factoryFilial,
        Factory\EnderecoFilial $factoryEnderecoFilial,
        Factory\ParceiroNegocio $factoryParceiroNegocio,
        Factory\Parceria $factoryParceria,
        Factory\Operador $factoryOperador,
        Factory\PerfilAcesso $factoryPerfilAcesso,
        Factory\PerfilAcessoParceiro $factoryPerfilAcessoParceiro,
        Factory\ParametroGeral $factoryParametroGeral,
        Factory\OrganizacaoOperador $factoryOrganizacaoOperador,
        Service\Operador $serviceOperador,
        \Teknisa\Libs\Util\Environment $environment,
        Service\Email\EmailSolicitacao $serviceEmailSolicitacao,
        Factory\Estruturam $factoryEstruturam,
        Factory\Hierarquiaorg $factoryHierarquiaorg,
        Factory\Estruturaoper $factoryEstruturaoper,
        Factory\OrganizationParameters $factoryOrganizationParameters,
        Factory\IpRestriction $factoryIpRestriction,
        Factory\Cliente $factoryCliente,
        Factory\EndeClie $factoryEndeClie,
        Factory\EnderecoParc $factoryEnderecoParc,
        DBConnection $dbConnection,
        Utilities $utilities
    ){
        $this->utilities = $utilities;
        $this->environment = $environment;
        $this->dbConnection = $dbConnection;
        $this->entityManager = $entityManager;
        $this->senhaGeradaParaOperador = null;
        $this->serviceOperador = $serviceOperador;
        $this->serviceEmailSolicitacao = $serviceEmailSolicitacao;
        $this->factoryFilial = $factoryFilial;
        $this->factoryCliente = $factoryCliente;
        $this->factoryEndeClie = $factoryEndeClie;
        $this->factoryEnderecoParc = $factoryEnderecoParc;
        $this->factoryCompany = $factoryCompany;
        $this->factoryOperador = $factoryOperador;
        $this->factoryParceria = $factoryParceria;
        $this->factoryEstruturam = $factoryEstruturam;
        $this->factoryOrganizacao = $organizationFactory;
        $this->factoryPerfilAcesso = $factoryPerfilAcesso;
        $this->factoryHierarquiaorg = $factoryHierarquiaorg;
        $this->factoryIpRestriction = $factoryIpRestriction;
        $this->factoryEstruturaoper = $factoryEstruturaoper;
        $this->factoryParametroGeral = $factoryParametroGeral;
        $this->factoryEnderecoFilial = $factoryEnderecoFilial;
        $this->factoryParceiroNegocio = $factoryParceiroNegocio;
        $this->factoryInscricaoEstadual = $stateInscriptionFactory;
        $this->factoryInscricaoMunicipal = $factoryInscricaoMunicipal;
        $this->factoryOrganizacaoOperador = $factoryOrganizacaoOperador;
        $this->factoryPerfilAcessoParceiro = $factoryPerfilAcessoParceiro;
        $this->factoryOrganizationParameters = $factoryOrganizationParameters;
        $this->repositoryFilial = $this->entityManager->getRepository(Utilities::GERAL_ENTITY_DIR . 'Filial');
        $this->repositoryEmpresa = $this->entityManager->getRepository(Utilities::GERAL_ENTITY_DIR . 'Empresa');
        $this->repositoryCliente = $this->entityManager->getRepository(Utilities::GERAL_ENTITY_DIR . 'Cliente');
        $this->repositoryParceria = $this->entityManager->getRepository(Utilities::GERAL_ENTITY_DIR . 'Parceria');

        $this->repositoryParcnegocio = $this->entityManager->getRepository(Utilities::GERAL_ENTITY_DIR . 'Parcnegocio');

        $this->repositoryOperador = $this->entityManager->getRepository(Utilities::GERAL_ENTITY_DIR . 'Operador');
        $this->repositoryParamOrg = $this->entityManager->getRepository(Utilities::TEKNISA_ENTITY_DIR . 'OmParamOrg');
        $this->repositoryOmProduto = $this->entityManager->getRepository(Utilities::TEKNISA_ENTITY_DIR . 'OmProduto');
        $this->repositoryOrganizacao = $this->entityManager->getRepository(Utilities::GERAL_ENTITY_DIR . 'Organizacao');
        $this->repositoryEstruturaoper = $this->entityManager->getRepository(Utilities::GERAL_ENTITY_DIR . 'Estruturaoper');
        $this->repositoryOmProdutoOrg = $this->entityManager->getRepository(Utilities::TEKNISA_ENTITY_DIR . 'OmProdutoOrg');
        $this->repositoryOmProdutoOrgElemento = $this->entityManager->getRepository(Utilities::TEKNISA_ENTITY_DIR . 'OmProdutoOrgElemento');
        $this->repositoryOmProdutoElemento = $this->entityManager->getRepository(Utilities::TEKNISA_ENTITY_DIR . 'OmProdutoElemento');
        $this->repositoryInscricaoEstadual = $this->entityManager->getRepository(Utilities::GERAL_ENTITY_DIR . 'Inscrestad');
        $this->repositoryIpRestriction = $this->entityManager->getRepository(Utilities::TEKNISA_ENTITY_DIR . 'OmIpRestriction');
        $this->repositoryOrganizacaoOperador = $this->entityManager->getRepository(Utilities::GERAL_ENTITY_DIR . 'Organizacaooper');
    }

    public function obtemTodos($filters, $page, $pageSize, $orderBy, $isIntId = true, $isPrimary = null) {
        $redirectionFilter = '';
        foreach ($filters as $key => $condition) {
            if($condition['columnName'] === 'HAS_REDIRECTION') {
                $redirectionFilter = $this->buildRedirectionFilter($condition['value']);
                unset($filters[$key]);
            }
        }
        $conditions = Utilities::createQueryFiltered($filters);
        if($isPrimary && $isPrimary == "S"){
            $conditions .= "AND NRORGPRINCIPAL IS NOT NULL";
        } else if($isPrimary && $isPrimary == "P"){
            $conditions .= "AND NRORGPRINCIPAL IS NULL";
        }
        foreach($orderBy as $idx => $order) {
            $conditions .= ' ORDER BY ' . $idx . ' ' . $order;
        }
        $limit  = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $organizacoes = $this->repositoryOrganizacao->obtemTodasOrganizacoes($conditions, $limit, $offset, $redirectionFilter);
        if($isIntId) {
            foreach ($organizacoes as &$organizacao) {
                $organizacao['NRORG'] = intval($organizacao['NRORG']);
            }
        }
        return $organizacoes;
    }

    private function buildRedirectionFilter($value) {
        $notValue = $value == 'N' ? 'NOT' : '';
        $filter = " AND $notValue EXISTS (SELECT OPR.ID
                                            FROM OM_PRODUTO_ORG OPR
                                           WHERE OPR.NRORG_ID = O.NRORG
                                             AND (OPR.DSURLREDIRECT IS NOT NULL OR OPR.IDUTREDIRECTRC = 'S'))";
        return $filter;
    }

    private function buildRedirectionFilterWithElements($value) {
        if($value == 'S') {
            return " AND (OG.DSURLREDIRECT IS NOT NULL OR OG.IDUTREDIRECTRC = 'S')";
        } else {
            return " AND (OG.DSURLREDIRECT IS NULL AND OG.IDUTREDIRECTRC = 'N')";
        }
    }

    public function allOrganizationRelations($filters,$nrorg){
        $conditions = $filters ? Utilities::createQueryFiltered($filters) : "";
        return $this->repositoryOrganizacao->allOrganizationRelations($conditions,$nrorg);
    }

    public function saveOrganizationRelations($nrOrgPrincipal,$nrOrg){
        $conexao = $this->entityManager->getConnection();
        try {
            $conexao->beginTransaction();
            foreach($nrOrg as $org){
                $entidadeOrganizacao = $this->repositoryOrganizacao->find(array("nrorg"=>$org));
                $entidadeOrganizacao->setNrorgprincipal($nrOrgPrincipal);
                $this->entityManager->persist($entidadeOrganizacao);
            }
            $this->entityManager->flush();
            $conexao->commit();
            return $entidadeOrganizacao->getNrorg();
        } catch(\Exception $ex){
            $conexao->rollback();
            throw $ex;
        }
    }

    public function obtemTodosComElemento($filters, $page, $pageSize, $orderBy, $isIntId = true) {
        $redirectionFilter = '';
        foreach ($filters as $key => $condition) {
            if($condition['columnName'] === 'HAS_REDIRECTION') {
                $redirectionFilter = $this->buildRedirectionFilterWithElements($condition['value']);
                unset($filters[$key]);
            }
        }
        $conditions = Utilities::createQueryFiltered($filters);
        $firstOrderBy = true;
        foreach($orderBy as $idx => $order) {
            $ob = $firstOrderBy ? 'ORDER BY ' : ',';
            $conditions .= $firstOrderBy ? $ob. $idx . ' ' . $order . ', NRORG ' . $order : $ob. $idx . ' ' . $order;
            $firstOrderBy = false;
        }
        $limit  = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $organizacoes = $this->repositoryOrganizacao->obtemTodasOrganizacoesComElemento($conditions, $limit, $offset, $redirectionFilter);
        if($isIntId) {
            foreach ($organizacoes as &$organizacao) {
                $organizacao['NRORG'] = intval($organizacao['NRORG']);
            }
        }
        return $organizacoes;
    }

    public function obtemOrganizacoesComProduto($filters, $page, $pageSize, $orderBy, $isIntId = true) {
        $conditions = Utilities::createQueryFiltered($filters);
        $firstOrderBy = true;
        $ordenation = null;
        foreach($orderBy as $idx => $order) {
            $ob = $firstOrderBy ? 'ORDER BY ' : ',';
            $ordenation .= $ob. $idx . ' ' . $order;
            $firstOrderBy = false;
        }
        $limit  = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        $organizacoes = $this->repositoryOrganizacao->obtemOrganizacoesComProduto($conditions, $limit, $offset, $ordenation);
        if($isIntId) {
            foreach ($organizacoes as &$organizacao) {
                $organizacao['NRORG'] = intval($organizacao['NRORG']);
            }
        }
        return $organizacoes;
    }

    public function obtemPorId($organizationId){
        return $this->repositoryOrganizacao->obtemPorId($organizationId);
    }

    public function obtemPorCNPJ($cnpj){
        return $this->repositoryOrganizacao->findBy(array('nrinsjurempr' => $cnpj));
    }

    public function atualiza(
        $nrorg, $nmrazsocialorg, $nmorganizacao, $tipoorg, $nrinsjurempr, $idativo,
        $dsemailoper, $nmoperador,
        $dslogradempr, $nrlogradempr, $dscmplogempr, $nrcepempr, $cdpais, $sgestado, $cdmunicipio, $nmbairempr,
        $cdinscmuni, $nrinscesta, $nrParcNegocio,$idOrgSaas, $prepStatus, $nrOrgPrincipal, $isPrimary
    ){
        $related = $this->allOrganizationRelations(null,$nrorg);
        if(count($related) > 0 && $isPrimary == "S"){
            throw new \Exception(Messages::ERR_PRIMARY_ORGANIZATION_WITH_RELATED_ORGANIZATIONS);
        }
        $isSaas = $idOrgSaas == "S" ? true : false;
        $this->entityManager->clear();
        $defaultEntityManager = $this->entityManager;
        if($isSaas){
            $this->entityManager = $this->dbConnection->getOrgEntityManager($nrorg);
            $conexao = $this->entityManager->getConnection();
        }
        $conexaoDefault = $defaultEntityManager->getConnection();
        try {
            $existsOrgDB = false;
            if($isSaas) {
                try {
                    $conexao->beginTransaction();
                    $existsOrgDB = true;
                } catch (\Exception $e) {}
            }

            $conexaoDefault->beginTransaction();

            $organizationEntity = $this->repositoryOrganizacao->findOneBy(array('nrorg' => $nrorg));
            if($nrOrgPrincipal){
                $primaryOrganizationEntity = $this->repositoryOrganizacao->findOneBy(array('nrorg' => $nrOrgPrincipal));
                if($primaryOrganizationEntity && $primaryOrganizationEntity->getNrorgprincipal()) {
                    throw new \Exception(Messages::ERR_SECONDARY_ORGANIZATION_SAVE_AS_PRIMARY);
                }
            }
            $orgOperatorEmail = $organizationEntity->getDsemailoper();
            $nmacesso = $this->cortaEmailParaCriarNomeAcesso($dsemailoper);
            $organizationEntity = $this->factoryOrganizacao->atualiza(
                $organizationEntity, $nmrazsocialorg, $nmorganizacao, $tipoorg, $nrinsjurempr, $idativo,
                $dsemailoper, $nmoperador, $nmacesso,
                $dslogradempr, $nrlogradempr, $dscmplogempr, $nrcepempr, $cdpais, $sgestado, $cdmunicipio, $nmbairempr,
                $cdinscmuni, $nrinscesta, $nrParcNegocio, $idOrgSaas, $prepStatus, $nrOrgPrincipal
            );
            $defaultEntityManager->persist($organizationEntity);


            if(($organizationEntity->getPrepstatus() == 'P' || $organizationEntity->getPrepstatus() == 'Z') && $isSaas) {
                $dbUser = Utilities::getOrgDbUserName($nrorg);
                $orgOperators = null;
                try {
                    $orgOperators = $this->repositoryOperador->findOrgOperator($orgOperatorEmail, $nrorg, $this->entityManager);
                } catch(\Exception $e) { }

                if(!is_null($orgOperators) && count($orgOperators) == 1) {
                    $cdoperador = $orgOperators[0]['CDOPERADOR'];
                    $this->repositoryOperador->updateOrgOperator($dbUser, $dsemailoper, $nmacesso, $nmoperador, $nrorg, $cdoperador, $this->entityManager);
                    /** @var GeralEntities\Operador $operador */
                    $operador = $this->repositoryOperador->findOneBy(array('dsemailoper' => $orgOperatorEmail,
                        'nrorg' => $this->environment->getCurrentOrganizationId()));
                    if(is_object($operador)) {
                        $operador->setDsemailoper($dsemailoper);
                        $operador->setNmacesso($nmacesso);
                        $operador->setNmoperador($nmoperador);
                        $defaultEntityManager->persist($operador);
                    }
                }
            }

            if(!$nrParcNegocio) {
                $this->createClient($organizationEntity);
            }

            if($existsOrgDB) {
                $this->entityManager->flush();
                $conexao->commit();
            }



            $defaultEntityManager->flush();
            $defaultEntityManager->commit();
            return true;
        } catch(\Exception $ex){
            if($isSaas && $conexao->isTransactionActive()) {
                $conexao->rollback();
            }
            if($conexaoDefault->isTransactionActive()) {
                $conexaoDefault->rollback();
            }
            throw $ex;
        }
    }

    public function cria($nmrazsocialorg, $nmorganizacao, $tipoorg, $nrinsjurempr, $idativo, $dsemailoper, $nmoperador,$dslogradempr,
                         $nrlogradempr, $dscmplogempr, $nrcepempr, $cdpais, $sgestado, $cdmunicipio, $nmbairro, $cdinscmuni, $nrinscesta, $nrParcNegocio, $idOrgSaas, $nrOrgPrincipal){
        $conexao = $this->entityManager->getConnection();

        try {
            $conexao->beginTransaction();
            if($nrOrgPrincipal){
                $primaryOrganizationEntity = $this->repositoryOrganizacao->findOneBy(array('nrorg' => $nrOrgPrincipal));
                if($primaryOrganizationEntity && $primaryOrganizationEntity->getNrorgprincipal()) {
                    throw new \Exception(Messages::ERR_SECONDARY_ORGANIZATION_SAVE_AS_PRIMARY);
                }
            }
            $entidadeOrganizacao = $this->saveOrganization($nmrazsocialorg, $nmorganizacao, $tipoorg, $nrinsjurempr, $idativo, $dsemailoper,
                $nmoperador,$dslogradempr, $nrlogradempr, $dscmplogempr, $nrcepempr, $cdpais,
                $sgestado, $cdmunicipio, $nmbairro, $cdinscmuni, $nrinscesta, $nrParcNegocio, $idOrgSaas, $nrOrgPrincipal);
            $this->entityManager->persist($entidadeOrganizacao);


            $this->entityManager->flush();

            if(!$nrParcNegocio) {
                $this->createClient($entidadeOrganizacao);
            }

            $conexao->commit();
            return $entidadeOrganizacao->getNrorg();
        } catch(\Exception $ex){
            $conexao->rollback();
            throw $ex;
        }
    }

    public function saveOrganization($nmrazsocialorg, $nmorganizacao, $tipoorg, $nrinsjurempr, $idativo,$dsemailoper, $nmoperador, $dslogradempr,
                                     $nrlogradempr, $dscmplogempr, $nrcepempr, $cdpais, $sgestado, $cdmunicipio, $nmbairro, $cdinscmuni, $nrinscesta, $nrParcNegocio = null, $idOrgSaas = 'S', $nrOrgPrincipal = null){

        $idOrganizacao = $this->repositoryOrganizacao->retornaMaxOrganizacao();
        $nrinscricaoorg = $this->validaInscricaoJuridica($nrinsjurempr, $idOrganizacao);
        $nrinscricaoorg = Utilities::removeCaracteresEspeciais($nrinscricaoorg);
        $nmacesso = $this->cortaEmailParaCriarNomeAcesso($dsemailoper);

        return $this->factoryOrganizacao->cria( $idOrganizacao, $nmrazsocialorg, $nmorganizacao, $tipoorg, $nrinscricaoorg, $nrinsjurempr,
            $idativo, $dsemailoper, $nmoperador, $nmacesso, $dslogradempr, $nrlogradempr, $dscmplogempr,
            $nrcepempr, $cdpais, $sgestado, $cdmunicipio, $nmbairro, $cdinscmuni, $nrinscesta, $nrParcNegocio, $idOrgSaas, $nrOrgPrincipal);
    }

    private function cortaEmailParaCriarNomeAcesso($email){
        $antesArroba = explode("@", $email)[0];
        $antesArroba = substr($antesArroba, 0, 28);
        return $antesArroba;
    }

    private function validaInscricaoJuridica($inscriptionNumber, $defaultValue){
        if(empty($inscriptionNumber)){
            $inscriptionNumber = str_pad($defaultValue, 20, "0", STR_PAD_LEFT);
        }
        return $inscriptionNumber;
    }

    public function removeVarios($organizations){
        try {
            foreach($organizations as  $organization){
                $this->removePorId($organization['NRORG'], false);
            }

            $this->entityManager->flush();
        } catch(\Exception $ex){
            throw $ex;
        }
    }

    public function removePorId($idOrganizacao, $flush = true){
        try {
            $entidadeOrganizacao = $this->repositoryOrganizacao->findOneBy(array('nrorg' => $idOrganizacao));

            if($this->estaPreparada($entidadeOrganizacao)){
                throw new \Exception("Não é possível remover uma organização depois de preparada.");
            }

            if($entidadeOrganizacao){
                $this->entityManager->remove($entidadeOrganizacao);
            }

            if($flush){
                $this->entityManager->flush();
            }
        } catch(\Exception $ex){
            throw $ex;
        }
    }

    private function estaPreparada(GeralEntities\Organizacao $entidadeOrganizacao){
        $empresa = $this->repositoryEmpresa->findOneBy(array('nrorg' => $entidadeOrganizacao->getNrorg()));
        return !(empty($empresa));
    }

    public function activateNotSaas($id, $nmoperador, $dsemailoper){
        $defaultEntityManager = $this->entityManager;
        $conexaoDefault = $defaultEntityManager->getConnection();
        $this->senhaGeradaParaOperador = null;
        $this->senhaGeradaParaOperador = $this->serviceOperador->geraSenhaAleatoria();
        $cdsenhaoperweb = md5($this->senhaGeradaParaOperador);
        try {
            $conexaoDefault->beginTransaction();
            /** @var GeralEntities\Organizacao $entidade */
            $entidade = $this->repositoryOrganizacao->findOneBy(array('nrorg' => $id));
            $isCreateCustomerPortalOperator = true;
            if($entidade->isSecundaria()) {
                /** @var GeralEntities\Organizacao $entidadePrincipal */
                $entidadePrincipal = $this->repositoryOrganizacao->findOneBy(array('nrorg' => $entidade->getNrorgprincipal()));
                if($entidadePrincipal->isSaas() && ($entidadePrincipal->getPrepstatus() != GeralEntities\Organizacao::BASE_PREPARADA &&
                                                    $entidadePrincipal->getPrepstatus() != GeralEntities\Organizacao::BASE_PREPARADA_ZMART)) {
                    throw new \Exception(Messages::ERR_PRIMATY_ORGANIZATION_NOT_PREPARED);
                } else if(!$entidadePrincipal->isSaas() && ($entidadePrincipal->getPrepstatus() != GeralEntities\Organizacao::BASE_PREPARADA_LOCAL_HOSTING &&
                                                            $entidadePrincipal->getPrepstatus() != GeralEntities\Organizacao::BASE_PREPARADA_ZMART_LOCAL_HOSTING)) {
                    throw new \Exception(Messages::ERR_PRIMATY_ORGANIZATION_NOT_PREPARED);
                }

                $validateOperatorAlreadyCreated = $this->repositoryOrganizacao->validateOperatorAlreadyCreated($entidade->getNrorgprincipal(), $dsemailoper);
                if(!empty($validateOperatorAlreadyCreated)){
                    $isCreateCustomerPortalOperator = false;
                }
            }
            $orgCliente = null;
            $operador = $this->repositoryOperador->findOneBy(array('dsemailoper' => $dsemailoper));
            if(!empty($operador)) {
                $cliente    = $this->repositoryCliente->findOneBy(array('cdcliente' => $operador->getCdcliente()));
                $orgCliente = $this->repositoryOrganizacao->findOneBy(array('nrparcnegocio' => $cliente->getNrparcnegocio()));
                if($orgCliente->getNrorg() == $entidade->getNrorg() || $orgCliente->getNrorg() == $entidade->getNrorgprincipal()) {
                    $isCreateCustomerPortalOperator = false;
                }
            }

            if($isCreateCustomerPortalOperator) {
                $this->insertCustomerPortalOperator($defaultEntityManager, $id, $entidade, $cdsenhaoperweb, $nmoperador, $dsemailoper);
            }
            $this->defineStatusPreparado($defaultEntityManager, $entidade);
            $defaultEntityManager->flush();
            $conexaoDefault->commit();
        } catch (\Exception $ex) {
            $conexaoDefault->rollBack();
            if(strpos($ex->getMessage(), 'RESOURCE_T_EMAIL_UK')) {
                $message = Messages::ERR_OPERATOR_RELATED_OTHER_ORGANIZATION;
                if($orgCliente) {
                   $message .= '<br>' . $orgCliente->getNrorg() . ' - ' . $orgCliente->getNmorganizacao();
                }
                throw new \Exception($message);
            } else {
                throw $ex;
            }
        }
    }

    public function prepara($id) {
        $parameters = InstanceProvider::getRepository()->getOmParamOrg()->getOrganizationParameters($id);
        $dbUser     = empty($parameters) || empty($parameters[0]['DSUSUARIOBANCO']) ? null : $parameters[0]['DSUSUARIOBANCO'];

        $defaultEntityManager = $this->entityManager;
        $this->entityManager = $this->dbConnection->getOrgEntityManager($id, false, false, $dbUser);
        $conexao = $this->entityManager->getConnection();
        $conexaoDefault = $defaultEntityManager->getConnection();

        try {
            $conexao->beginTransaction();
            $conexaoDefault->beginTransaction();

            /** @var GeralEntities\Organizacao $entidade */
            $entidade = $this->repositoryOrganizacao->findOneBy(array('nrorg' => $id));

            $this->validaGeracaoDadosInicias($entidade);

            $this->preencheTabelaEmpresa($entidade);
            $this->entityManager->flush();

            $this->preencheTabelaInscrestad($entidade);
            $this->preencheTabelaInscrmunicip($entidade);
            $this->entityManager->flush();

            $filial = $this->preencheTabelaFilial($entidade);
            $estruturaIds = $this->preencheTabelaEstruturam($entidade);
            $this->entityManager->flush();

            $this->preencheTabelaEndefili($entidade);

            $nrparcnegocio = $this->preencheTabelaParcnegocio($entidade);
            $this->preencheTabelaHierarquiaorg($entidade, $estruturaIds);
            $this->entityManager->flush();

            $this->preencheTabelaParceria($entidade, $nrparcnegocio);
            $operador = $this->preencheTabelaOperador($entidade, $estruturaIds[0], $nrparcnegocio);
            $nrperfilacesso = $this->preencheTabelaPerfilacesso($entidade);
            $this->entityManager->flush();

            $this->preencheTabelaFilioper($operador->getCdoperador(), $filial->getCdfilial(), $id);
            $this->preencheTabelaPerfacessparc($entidade, $nrparcnegocio, $nrperfilacesso);

            // Phillipe Silva 24/09/21 - Tabela PARAMGERAL passou a ser importada pelos Pré Dados Iniciais.
            // Dessa forma, não é mais necessário inserir na geração dos dados iniciais.
            // $this->preencheTabelaParamgeral($entidade);
            $this->preencheTabelaParamfilial($entidade, $filial->getCdfilial());
            $this->preencheTabelasEstoque($entidade, $filial->getCdfilial());
            $this->preencheTabelasLoja($entidade, $filial->getCdfilial());

            $nrorganizacaooper = $this->preencheTabelaOrganizacaooper($entidade, $operador->getCdoperador());
            $this->entityManager->flush();

            $this->preencheTabelaEstruturaoper($entidade, $nrorganizacaooper, $estruturaIds[1]);
            $this->defineStatusPreparado($defaultEntityManager, $entidade);

            $this->insertCustomerPortalOperator($defaultEntityManager, $id, $entidade, $operador->getCdsenhaoperweb(),
                $operador->getNmoperador(), $operador->getDsemailoper());

            $this->entityManager->flush();
            $defaultEntityManager->flush();
            $conexao->commit();
            $conexaoDefault->commit();

            $this->enviaEmailComSenhaGerada($entidade);
            return true;
        } catch(\Exception $ex){
            if($conexao->isTransactionActive()) {
                $conexao->rollback();
            }
            if($conexaoDefault->isTransactionActive()) {
                $conexaoDefault->rollback();
            }
            throw $ex;
        }
    }

    private function preencheTabelaEmpresa(GeralEntities\Organizacao $entidadeOrganizacao){
        $nrorg = $entidadeOrganizacao->getNrorg();
        $cdempresa = '01';
        $nmempresa = $entidadeOrganizacao->getNmorganizacao();
        $nmrazsocempr = $entidadeOrganizacao->getNmrazsocialorg();
        $cdpais = $entidadeOrganizacao->getCdpais();
        $sgestado = $entidadeOrganizacao->getSgestado();
        $cdmunicipio = $entidadeOrganizacao->getCdmunicipio();
        $nmbairempr = $entidadeOrganizacao->getNmbairempr();
        $nrcepempr = $entidadeOrganizacao->getNrcepempr();
        $dslogradempr = $entidadeOrganizacao->getDslogradempr();
        $nrlogradempr = $entidadeOrganizacao->getNrlogradempr();
        $dsendeempr = $dslogradempr . ', ' . $nrlogradempr;
        $idtpijurempr = $entidadeOrganizacao->getIdtipoinscrorg();
        $nrinsjurempr = $entidadeOrganizacao->getNrinsjurempr();
        $nrinscesta = $entidadeOrganizacao->getNrinscesta();
        $cdinscmuni = $entidadeOrganizacao->getCdinscmuni();
        $dsemailempresa = $entidadeOrganizacao->getDsemailempresa();
        $cdoperinclusao = $this->environment->getCurrentUserId();

        $entity = $this->factoryCompany->cria(
            $cdempresa, $nrorg, $nmempresa, $nmrazsocempr,
            $cdpais, $sgestado, $cdmunicipio, $nmbairempr, $nrcepempr,
            $dsendeempr, $dslogradempr, $nrlogradempr,
            $idtpijurempr, $nrinsjurempr, $nrinscesta, $cdinscmuni,
            $dsemailempresa,
            $cdoperinclusao
        );

        $this->entityManager->persist($entity);
    }

    private function preencheTabelaInscrestad(GeralEntities\Organizacao $entidadeOrganizacao){
        $nrorg = $entidadeOrganizacao->getNrorg();
        $cdempresa = '01';
        $idativo = $entidadeOrganizacao->getIdativo();

        $idtpijuriest = $entidadeOrganizacao->getIdtipoinscrorg();
        $nrinsjuriest = Utilities::removeCaracteresEspeciais($entidadeOrganizacao->getNrinsjurempr());
        $cdinscesta = $entidadeOrganizacao->getNrinscesta();
        $cdinscesta = empty($cdinscesta) ? 'ISENTA' : Utilities::removeCaracteresEspeciais($cdinscesta);
        $idapurlucpre = 'N';
        $idcreddebnorm = 'N';
        $idsugregstentr = 'N';
        $idtpimpresdanfe = '1';
        $idtpambientenfe = '2';
        $idutserecfnf = 'N';
        $idutnrcupomnf = 'N';
        $idutcargatrib = 'N';

        $cdpais = $entidadeOrganizacao->getCdpais();
        $sgestado = $entidadeOrganizacao->getSgestado();
        $cdmunicipio = $entidadeOrganizacao->getCdmunicipio();
        $dsendeiest = $this->criaTextoComEnderecoCompleto(
            $entidadeOrganizacao->getDslogradempr(), $entidadeOrganizacao->getNrlogradempr(),
            $entidadeOrganizacao->getDscmplogempr()
        );
        $nmbairiest = $entidadeOrganizacao->getNmbairempr();
        $nrcepiest = $entidadeOrganizacao->getNrcepempr();
        $nrteleiest = $entidadeOrganizacao->getNrtelefoempr();

        $dtinclusao = new SetformsDateTime();
        $cdoperinclusao = $this->environment->getCurrentUserId();
        $nrorginclusao = $this->environment->getCurrentOrganizationId();

        $entity = $this->factoryInscricaoEstadual->cria(
            $nrorg, $cdempresa, $idativo,
            $idtpijuriest, $nrinsjuriest, $cdinscesta, $idapurlucpre, $idcreddebnorm, $idsugregstentr,
            $idtpimpresdanfe, $idtpambientenfe, $idutserecfnf, $idutnrcupomnf, $idutcargatrib,
            $cdpais, $sgestado, $cdmunicipio, $nmbairiest, $nrcepiest, $dsendeiest,
            $nrteleiest,
            $dtinclusao, $cdoperinclusao, $nrorginclusao
        );

        $this->entityManager->persist($entity);
    }

    private function preencheTabelaInscrmunicip(GeralEntities\Organizacao $entidadeOrganizacao){
        $nrorg = $entidadeOrganizacao->getNrorg();
        $cdempresa = '01';
        $cdinscmuni = $entidadeOrganizacao->getCdinscmuni();
        $cdinscmuni = empty($cdinscmuni) ? 'ISENTA' : Utilities::removeCaracteresEspeciais($cdinscmuni);
        $dtinclusao = new SetformsDateTime();
        $idambtrabnfse = '2';

        $cdoperinclusao = $this->environment->getCurrentUserId();
        $nrorginclusao = $this->environment->getCurrentOrganizationId();

        $entity = $this->factoryInscricaoMunicipal->cria(
            $nrorg, $cdempresa, $cdinscmuni, $dtinclusao, $idambtrabnfse,
            $cdoperinclusao, $nrorginclusao
        );

        $this->entityManager->persist($entity);
    }

    private function preencheTabelaFilial(GeralEntities\Organizacao $entidadeOrganizacao){
        $cdfilial = '0001';
        $nmfilial = $entidadeOrganizacao->getNmorganizacao();
        $cdempresa = '01';
        $cdinscmuni = $entidadeOrganizacao->getCdinscmuni();
        $cdinscmuni = empty($cdinscmuni) ? 'ISENTA' : Utilities::removeCaracteresEspeciais($cdinscmuni);
        $cdinscesta = $entidadeOrganizacao->getNrinscesta();
        $cdinscesta = empty($cdinscesta) ? 'ISENTA' : Utilities::removeCaracteresEspeciais($cdinscesta);
        $nrinsjurfili = $entidadeOrganizacao->getNrinsjurempr();
        $idendemisfil = 'P';
        $idendcfopfil = 'P';
        $idutfilipadr = 'S';
        $idcumupiscofil = 'S';
        $idcomparqxmlfil = 'N';
        $idcntrresc = 'N';
        $idconsorcfil = 'N';
        $nrorg = $entidadeOrganizacao->getNrorg();
        $dtinclusao = new SetformsDateTime();
        $idativo = $entidadeOrganizacao->getIdativo();
        $idcalcomestorc = 'N';

        $cdoperinclusao = $this->environment->getCurrentUserId();
        $nrorginclusao = $this->environment->getCurrentOrganizationId();

        $entity = $this->factoryFilial->cria(
            $cdfilial, $nmfilial, $cdempresa, $nrorg, $idativo,
            $cdinscmuni, $cdinscesta, $nrinsjurfili, $idendemisfil, $idendcfopfil,
            $idutfilipadr, $idcomparqxmlfil, $idcntrresc, $idconsorcfil, $idcalcomestorc,
            $dtinclusao, $cdoperinclusao, $nrorginclusao, $idcumupiscofil
        );

        $this->entityManager->persist($entity);

        return $entity;
    }

    private function preencheTabelaFilioper($cdoperador, $cdfilial, $nrorg) {
        $cdoperinclusao = $this->environment->getCurrentUserId();
        $nrorginclusao = $this->environment->getCurrentOrganizationId();
        $dtinclusao = new SetformsDateTime();

        $entity = $this->factoryFilial->criaFilioper($cdoperador, $cdfilial, $nrorg, $dtinclusao, $cdoperinclusao, $nrorginclusao);

        $this->entityManager->persist($entity);
    }

    private function preencheTabelaEstruturam(GeralEntities\Organizacao $entidadeOrganizacao) {
        $nrorg         = $entidadeOrganizacao->getNrorg();
        $dtinivigencia = new SetformsDateTime();
        $dtinclusao    = new SetformsDateTime();
        $idativo       = 'S';

        $repositoryEstruturam = InstanceProvider::getRepository()->getEstruturam($this->entityManager);

        // Primeira estrutura
        $cdintestrutura = '0001'; // CDFILIAL
        $primeiraEstrutura = $repositoryEstruturam->findOneBy(array('cdintestrutura' => $cdintestrutura));

        if(empty($primeiraEstrutura)) {
            $nrtipoestrutura = 2; // FILIAL
            $primeiraEstrutura = $this->factoryEstruturam->cria(
                $nrorg, $nrtipoestrutura, $idativo, $cdintestrutura,
                $dtinivigencia, $dtinclusao
            );
            $this->entityManager->persist($primeiraEstrutura);
        }

        // Segunda estrutura
        $cdintestrutura = '01'; // CDEMPRESA
        $segundaEstrutura = $repositoryEstruturam->findOneBy(array('cdintestrutura' => $cdintestrutura));

        if(empty($segundaEstrutura)) {
            $nrtipoestrutura = 1; // EMPRESA
            $segundaEstrutura = $this->factoryEstruturam->cria(
                $nrorg, $nrtipoestrutura, $idativo, $cdintestrutura,
                $dtinivigencia, $dtinclusao
            );
            $this->entityManager->persist($segundaEstrutura);
        }
        return array($primeiraEstrutura->getNrestruturam(), $segundaEstrutura->getNrestruturam());
    }

    private function preencheTabelaEndefili(GeralEntities\Organizacao $entidadeOrganizacao){
        $nrorg = $entidadeOrganizacao->getNrorg();
        $cdfilial = '0001';
        $idtpendefili = 'P';
        $idativo = $entidadeOrganizacao->getIdativo();

        $cdpais = $entidadeOrganizacao->getCdpais();
        $sgestado = $entidadeOrganizacao->getSgestado();
        $cdmunicipio = $entidadeOrganizacao->getCdmunicipio();
        $dsendefili = $this->criaTextoComEnderecoCompleto(
            $entidadeOrganizacao->getDslogradempr(), $entidadeOrganizacao->getNrlogradempr(),
            $entidadeOrganizacao->getDscmplogempr()
        );
        $nmbairfili = $entidadeOrganizacao->getNmbairempr();
        $nrcepfili = $entidadeOrganizacao->getNrcepempr();
        $nrtelefili = $entidadeOrganizacao->getNrtelefoempr();

        $iduttlsemail = 'N';
        $idutsslemail = 'N';

        $dtultatuefil = new SetformsDateTime();
        $dtinclusao = new SetformsDateTime();
        $cdoperinclusao = $this->environment->getCurrentUserId();
        $nrorginclusao = $this->environment->getCurrentOrganizationId();

        $entity = $this->factoryEnderecoFilial->cria(
            $nrorg, $cdfilial, $idtpendefili, $idativo,
            $cdpais, $sgestado, $cdmunicipio, $dsendefili, $nmbairfili, $nrcepfili, $nrtelefili,
            $iduttlsemail, $idutsslemail, $dtultatuefil, $dtinclusao, $cdoperinclusao, $nrorginclusao
        );

        $this->entityManager->persist($entity);
    }

    private function criaTextoComEnderecoCompleto($rua, $numero = '', $complemento = ''){
        $enderecoCompleto = $rua;
        if( ! empty($numero)){
            $enderecoCompleto = $enderecoCompleto . ', ' . $numero;
        }
        if( ! empty($complemento)){
            $enderecoCompleto = $enderecoCompleto . ' - ' . $complemento;
        }
        return $enderecoCompleto;
    }

    private function preencheTabelaParcnegocio(GeralEntities\Organizacao $entidadeOrganizacao,
                                               $cdtipoparcprincipal = 'OPERADOR', $nrorg = null, $nrorgZmart = null) {

        $currentNrorg = $nrorgZmart ? $nrorgZmart : $entidadeOrganizacao->getNrorg();

        $nrparcnegocio  = Utilities::generateNewCode($currentNrorg, "PARCNEGOCIO",  1, 10, true, false, $this->entityManager->getConnection());
        if(empty($nrorg)) {
            $nrorg = $entidadeOrganizacao->getNrorg();
        }

        $nmprincipalparc = $entidadeOrganizacao->getNmrazsocialorg();
        $nmsecundariparc = $entidadeOrganizacao->getNmorganizacao();
        $idativo = 'S';

        $nrinscricaoparc = Utilities::removeCaracteresEspeciais($entidadeOrganizacao->getNrinsjurempr());
        $cdtipoinscricao = $this->checaTipoInscricaoJuridica($entidadeOrganizacao->getNrinsjurempr());
        $idpessoafisica = $this->checaSePessoaFisicaPorInscricaoJuridica($entidadeOrganizacao->getNrinsjurempr());
        $idinstituicao = $this->checaSeInstituicaoPorInscricaoJuridica($entidadeOrganizacao->getNrinsjurempr());
        $idparcfundido = 'N';

        $dtinclusao = new SetformsDateTime();
        $dtultatu = new SetformsDateTime();
        $cdoperinclusao = $this->environment->getCurrentUserId();
        $nrorginclusao = $this->environment->getCurrentOrganizationId();

        $repositoryParcnegocio = InstanceProvider::getRepository()->getParcnegocio($this->entityManager);
        if(empty($repositoryParcnegocio->findOneBy(array('nrparcnegocio' => $nrparcnegocio)))) {
            $entidadeParceiroNegocio = $this->factoryParceiroNegocio->cria(
                $nrorg, $nrparcnegocio, $nmprincipalparc, $nmsecundariparc, $idativo,
                $nrinscricaoparc, $cdtipoparcprincipal, $cdtipoinscricao, $idpessoafisica,
                $idinstituicao, $idparcfundido,
                $dtinclusao, $dtultatu, $cdoperinclusao, $nrorginclusao
            );
            $this->entityManager->persist($entidadeParceiroNegocio);
        }

        return $nrparcnegocio;
    }

    private function preencheTabelaCliente(GeralEntities\Organizacao $entidadeOrganizacao, $nrparcnegocio, $nrorg = null){
        if(empty($nrorg)) {
            $nrorg = $entidadeOrganizacao->getNrorg();
        }
        $nmrazsocclie = substr($entidadeOrganizacao->getNmrazsocialorg(), 0, 49);
        $nmfantclie   = substr($entidadeOrganizacao->getNmorganizacao(), 0, 49);
        $idativo      = 'S';
        $nrinsjurclie = Utilities::removeCaracteresEspeciais($entidadeOrganizacao->getNrinsjurempr());
        $idtpijurclie = $entidadeOrganizacao->getIdtipoinscrorg();
        $cdcliente    = $entidadeOrganizacao->getIdtipoinscrorg() . $nrinsjurclie;

        $dtinclusao     = new SetformsDateTime();
        $cdoperinclusao = $this->environment->getCurrentUserId();
        $nrorginclusao  = $this->environment->getCurrentOrganizationId();

        $entidadeCliente = $this->factoryCliente->cria(
            $nrorg, $nrparcnegocio, $cdcliente, $idtpijurclie, $idativo,
            $nrinsjurclie, $nmrazsocclie, $nmfantclie,
            $dtinclusao, $dtinclusao, $cdoperinclusao, $nrorginclusao
        );

        $this->entityManager->persist($entidadeCliente);
        $this->entityManager->flush();

        return $cdcliente;
    }

    private function preencheTabelaEnderecoParc(GeralEntities\Organizacao $entidadeOrganizacao, $nrparcnegocio,
                                                $nrenderecoparc, $nrorg = null, $cdtipoendereco = 'PRINCIPAL'){
        if(empty($nrorg)) {
            $nrorg = $entidadeOrganizacao->getNrorg();
        }
        $idativo = 'S';

        $cdpais      = $entidadeOrganizacao->getCdpais();
        $sgestado    = $entidadeOrganizacao->getSgestado();
        $cdmunicipio = $entidadeOrganizacao->getCdmunicipio();

        $dsendereco       = $entidadeOrganizacao->getDslogradempr();
        $dscompleendereco = $entidadeOrganizacao->getDscmplogempr();
        $nmbairroendereco = $entidadeOrganizacao->getNmbairempr();
        $nrcependereco    = $entidadeOrganizacao->getNrcepempr();

        $dtinclusao     = new SetformsDateTime();
        $cdoperinclusao = $this->environment->getCurrentUserId();
        $nrorginclusao  = $this->environment->getCurrentOrganizationId();

        $entidadeCliente = $this->factoryEnderecoParc->cria(
            $nrorg, $cdtipoendereco, $nrparcnegocio, $idativo,
            $cdpais, $sgestado, $cdmunicipio, $dsendereco,
            $dscompleendereco, $nmbairroendereco, $nrcependereco,$nrenderecoparc,
            $dtinclusao, $dtinclusao, $cdoperinclusao, $nrorginclusao
        );
        $this->entityManager->persist($entidadeCliente);
    }

    private function preencheTabelaEndeClie(GeralEntities\Organizacao $entidadeOrganizacao, $cdcliente,
                                            $nrenderecoparc, $nrorg = null, $idtpendeclie = 'P'){
        if(empty($nrorg)) {
            $nrorg = $entidadeOrganizacao->getNrorg();
        }
        $idativo = 'S';

        $cdpais      = $entidadeOrganizacao->getCdpais();
        $sgestado    = $entidadeOrganizacao->getSgestado();
        $cdmunicipio = $entidadeOrganizacao->getCdmunicipio();

        $dsendeclie   = $entidadeOrganizacao->getDslogradempr();
        $dscompendcli = $entidadeOrganizacao->getDscmplogempr();
        $nmbairclie   = $entidadeOrganizacao->getNmbairempr();
        $nrcepclie    = $entidadeOrganizacao->getNrcepempr();

        $dtinclusao     = new SetformsDateTime();
        $cdoperinclusao = $this->environment->getCurrentUserId();
        $nrorginclusao  = $this->environment->getCurrentOrganizationId();

        $entidadeCliente = $this->factoryEndeClie->cria(
            $nrorg, $idtpendeclie, $cdcliente, $idativo,
            $cdpais, $sgestado, $cdmunicipio, $dsendeclie,
            $dscompendcli, $nmbairclie, $nrcepclie,$nrenderecoparc,
            $dtinclusao, $dtinclusao, $cdoperinclusao, $nrorginclusao
        );

        $this->entityManager->persist($entidadeCliente);
    }

    private function checaTipoInscricaoJuridica($numeroInscricao){
        $tipo = "LIVRE";
        if($this->checaSeCNPJ($numeroInscricao)){
            $tipo = "CNPJ";
        } elseif($this->checaSeCPF($numeroInscricao)){
            $tipo = "CPF";
        }
        return $tipo;
    }

    private function checaSePessoaFisicaPorInscricaoJuridica($numeroInscricao){
        $idPessoaFisica = 'N';
        if($this->checaSeCPF($numeroInscricao)){
            $idPessoaFisica = 'S';
        }
        return $idPessoaFisica;
    }

    private function checaSeInstituicaoPorInscricaoJuridica($numeroInscricao){
        $idInstituicao = 'S';
        if($this->checaSeCPF($numeroInscricao)){
            $idInstituicao = 'N';
        }
        return $idInstituicao;
    }

    private function checaSeCPF($numero){
        $numero = Utilities::removeCaracteresEspeciais($numero);
        return strlen($numero) == 11;
    }

    private function checaSeCNPJ($numero){
        $numero = Utilities::removeCaracteresEspeciais($numero);
        return strlen($numero) == 14;
    }

    private function preencheTabelaParceria(GeralEntities\Organizacao $entidadeOrganizacao, $nrparcnegocio, $cdtipoparceiro = "OPERADOR", $nrorg = null) {
        $nrparceria = Utilities::generateNewCode($entidadeOrganizacao->getNrorg(), "PARCERIA", 1, 10, true, false, $this->entityManager->getConnection());
        if(empty($nrorg)) {
            $nrorg = $entidadeOrganizacao->getNrorg();
        }
        $idativo = 'S';
        $dtinclusao = new SetformsDateTime();
        $dtultatu = new SetformsDateTime();
        $cdoperinclusao = $this->environment->getCurrentUserId();
        $nrorginclusao = $this->environment->getCurrentOrganizationId();

        $entity = $this->factoryParceria->cria(
            $nrorg, $nrparceria, $nrparcnegocio, $cdtipoparceiro, $idativo,
            $dtinclusao, $dtultatu, $cdoperinclusao, $nrorginclusao
        );
        $this->entityManager->persist($entity);
    }

    private function preencheTabelaOperador(GeralEntities\Organizacao $entidade, $nrestruttrab, $nrparcnegocio){
        $nrorg = $entidade->getNrorg();
        $nrorgtrab = $nrorg;
        $cdoperador = Utilities::preencheStringComZero($nrorg . '1', 12);
        $nmoperador = $entidade->getNmoperador();
        $dsemailoper = $entidade->getDsemailoper();
        $nmacesso = $entidade->getNmacesso();

        $this->senhaGeradaParaOperador = null;
        $this->senhaGeradaParaOperador = $this->serviceOperador->geraSenhaAleatoria();
        $cdsenhaoperweb = md5($this->senhaGeradaParaOperador);
        $cdsenhoper = Utilities::geraCriptografiaSenhaDelphi($this->senhaGeradaParaOperador);

        $cdpais = $entidade->getCdpais();
        $sgestado = $entidade->getSgestado();
        $cdmunicipio = $entidade->getCdmunicipio();

        $idatuoperador = 'S';
        $dtinclusao = new SetformsDateTime();
        $idcadpraprovore = 'N';
        $idcadprproceore = 'N';

        $cdoperinclusao = $this->environment->getCurrentUserId();
        $nrorginclusao = $this->environment->getCurrentOrganizationId();
        $this->emailOperadorCriado = $dsemailoper;
        $this->codigoOperadorCriado = $cdoperador;
        $this->nomeOperadorCriado = $nmoperador;
        $entidade = $this->factoryOperador->cria(
            $nrorg, $nrorgtrab, $nrestruttrab, $cdoperador, $nmoperador, $dsemailoper,
            $nmacesso, $cdsenhaoperweb, $cdsenhoper,
            $cdpais, $sgestado, $cdmunicipio,
            $idatuoperador, $dtinclusao, $idcadpraprovore, $idcadprproceore, $nrparcnegocio,
            $cdoperinclusao, $nrorginclusao
        );

        $this->entityManager->persist($entidade);
        return $entidade;
    }

    private function preencheTabelaHierarquiaorg(GeralEntities\Organizacao $entidadeOrganizacao, $estruturaIds){
        $nrhierarquiaorg = 1;
        $nrorg = $entidadeOrganizacao->getNrorg();
        $nrtipohierarqorg = 1;
        $nrestruturamrel = $estruturaIds[1];
        $nrestruturamsup = $estruturaIds[0];
        $dtinivigencia = new SetformsDateTime();
        $dtinclusao = new SetformsDateTime();

        $entidadeHierarquiaorg = $this->factoryHierarquiaorg->cria(
            $nrhierarquiaorg, $nrorg, $nrtipohierarqorg, $nrestruturamrel, $nrestruturamsup,
            $dtinivigencia, $dtinclusao
        );

        $this->entityManager->persist($entidadeHierarquiaorg);
    }

    private function preencheTabelaPerfilacesso(GeralEntities\Organizacao $entidadeOrganizacao, $incrementCode = false) {
        $nrperfilacesso = Utilities::generateNewCode($entidadeOrganizacao->getNrorg(), "PERFILACESSO", 1, 10, true, false, $this->entityManager->getConnection());
        $nrperfilacesso = $incrementCode ? $nrperfilacesso + 10 : $nrperfilacesso;
        $nrorg          = $entidadeOrganizacao->isSecundaria() ? $entidadeOrganizacao->getNrorgprincipal() : $entidadeOrganizacao->getNrorg();
        $nmperfilacesso = 'Administrador de sistema';
        $idativo        = 'S';
        $dtinclusao     = new SetformsDateTime();
        $cdoperinclusao = $this->environment->getCurrentUserId();
        $nrorginclusao  = $this->environment->getCurrentOrganizationId();

        $entidadePerfilAcesso = $this->factoryPerfilAcesso->cria(
            $nrorg,
            $nrperfilacesso, $nmperfilacesso, $idativo,
            $dtinclusao, $cdoperinclusao, $nrorginclusao
        );

        $this->entityManager->persist($entidadePerfilAcesso);
        return $nrperfilacesso;
    }

    private function preencheTabelaOperClie($cdOperador, $cdCliente) {
        $dtinclusao      = new SetformsDateTime();
        $cdoperinclusao  = $this->environment->getCurrentUserId();
        $nrorginclusao   = $this->environment->getCurrentOrganizationId();
        $currentNrorg    = $this->environment->getCurrentOrganizationId();

        $entidadeOperClie = InstanceProvider::getFactory()->getOperclie()->cria(
            $cdOperador, $cdCliente, 'C', 'N', $currentNrorg, $dtinclusao, $cdoperinclusao, $nrorginclusao
        );
        return $entidadeOperClie;
    }

    private function preencheTabelaPerfacessparc(GeralEntities\Organizacao $entidadeOrganizacao, $nrparcnegocio,
                                                 $nrperfilacesso, EntityManager $defaultEntityManager = NULL, $nrorgZmart = NULL) {
        $nrorg           = $entidadeOrganizacao->isSecundaria() ? $entidadeOrganizacao->getNrorgprincipal() : $entidadeOrganizacao->getNrorg();
        $currentNrorg    = $nrorgZmart ? $nrorgZmart : $nrorg;
        $nrperfacessparc = Utilities::generateNewCode($currentNrorg, "PERFACESSPARC", 1, 10, true, false, $this->entityManager->getConnection());
        $idativo         = 'S';
        $dtinclusao      = new SetformsDateTime();
        $cdoperinclusao  = $this->environment->getCurrentUserId();
        $nrorginclusao   = $this->environment->getCurrentOrganizationId();
        $currentNrorg    = $this->environment->getCurrentOrganizationId();

        if ($defaultEntityManager) {
            $perfilAcessoAdm = [];
            $organizationRepository = InstanceProvider::getRepository()->getOrganization();
            $result = $organizationRepository->getAccessProfileFromClients($this->environment->getCurrentOrganizationId());
            if(!empty($result)) {
                foreach($result as $perfAcesso) {
                    array_push($perfilAcessoAdm, intval($perfAcesso['NRPERFILACESSO']));
                }
            } else {
                $perfilAcessoRepository = InstanceProvider::getRepository()->getPerfilAcesso();
                $result = $perfilAcessoRepository->findBy(array('idperfilacessoadm' => 'S', 'idativo' => 'S'));
                foreach($result as $perfAcesso) {
                    array_push($perfilAcessoAdm, $perfAcesso->getNrperfilacesso());
                }
            }

            foreach ($perfilAcessoAdm as $perfilAcesso) {
                /** @var GeralEntities\PerfilAcesso $perfilAcesso */
                $nrperfilacesso = $perfilAcesso;
                $nrperfacessparc = Utilities::generateNewCode($currentNrorg, "PERFACESSPARC", 1, 10, false, false, $defaultEntityManager->getConnection());
                $entidadePerfAcessParc = $this->factoryPerfilAcessoParceiro->cria(
                    $currentNrorg, $nrperfacessparc, $nrparcnegocio, $nrperfilacesso,
                    $idativo, $dtinclusao, $cdoperinclusao, $nrorginclusao
                );
                $defaultEntityManager->persist($entidadePerfAcessParc);
            }
        } else {
            $entidadePerfAcessParc = $this->factoryPerfilAcessoParceiro->cria(
                $nrorg, $nrperfacessparc, $nrparcnegocio, $nrperfilacesso,
                $idativo, $dtinclusao, $cdoperinclusao, $nrorginclusao
            );
            $this->entityManager->persist($entidadePerfAcessParc);
        }
    }

    private function preencheTabelaParamgeral(GeralEntities\Organizacao $entidadeOrganizacao){
        $nrorg = $entidadeOrganizacao->getNrorg();
        $cdpargeral = '0001';
        $idutCloud = 'S';
        $dtinclusao = new SetformsDateTime();
        $entidadeParametroGeral = $this->factoryParametroGeral->preparaParaOrganizacao($nrorg, $cdpargeral, $dtinclusao, $idutCloud);
        $this->entityManager->persist($entidadeParametroGeral);
    }

    private function preencheTabelaParamfilial(GeralEntities\Organizacao $entidadeOrganizacao, $cdfilial){
        $nrorg = $entidadeOrganizacao->getNrorg();
        $dtinclusao = new SetformsDateTime();
        $entidadeParametroFilial = InstanceProvider::getFactory()->getParametroFilial()->preparaParaOrganizacao($nrorg, $cdfilial, $dtinclusao);
        $this->entityManager->persist($entidadeParametroFilial);
    }

    private function preencheTabelasEstoque(GeralEntities\Organizacao $entidadeOrganizacao, $cdfilial) {
        $nrorg = $entidadeOrganizacao->getNrorg();
        $dtinclusao = new SetformsDateTime();
        $factory = InstanceProvider::getFactory()->getEstoque();

        $almoxarifado = $factory->criaAlmoxarifado($nrorg, $cdfilial, $dtinclusao);
        $this->entityManager->persist($almoxarifado);
        $this->entityManager->flush();

        $localEstoque = $factory->criaLocalEstoque($nrorg, $cdfilial, $dtinclusao);
        $this->entityManager->persist($localEstoque);
        
        $nmTabs = array('ALMOXARIFADO', 'LOCALESTOQUE', 'PARAMFILIAL', 'PARAMPEDIDO', 'PARAMPLANEJA', 'PARCMPSUP', 'PAREXPEDICAO');
        foreach($nmTabs as $nmtabparamfili) {            
            $paramFiliPadr = $factory->criaParamFiliPadr($nrorg, $cdfilial, $nmtabparamfili, $dtinclusao);
            $this->entityManager->persist($paramFiliPadr);
        }
        
        $setor = $factory->criaSetor($nrorg, '0001', 'P', 'Padrão', $dtinclusao);
        $this->entityManager->persist($setor);
    }

    private function preencheTabelasLoja(GeralEntities\Organizacao $entidadeOrganizacao, $cdfilial) {
        $factoryLoja = InstanceProvider::getFactory()->getLoja();
        $factoryCliente = InstanceProvider::getFactory()->getCliente();

        $nrorg = $entidadeOrganizacao->getNrorg();
        $dtinclusao = new SetformsDateTime();
        $cdloja = '01';
        $nmloja = 'ORGANIZAÇÃO REFERÊNCIA';
        
        $loja = $factoryLoja->criaLoja($nrorg, $cdfilial, $cdloja, $nmloja, $dtinclusao);
        $this->entityManager->persist($loja);
        
        $cdcliente = 'O99999';
        $nmrazsocclie = 'CLIENTE REFERÊNCIA';
        $clienfilial = $factoryCliente->cria($nrorg, null, $cdcliente, 'O', 'S', '99999', $nmrazsocclie, $nmrazsocclie, $dtinclusao, $dtinclusao);
        $this->entityManager->persist($clienfilial);
        $this->entityManager->flush();
        
        $cdcaixa = '001';
        $nmcaixa = 'ORGANIZAÇÃO REFERÊNCIA';
        $caixa = $factoryLoja->criaCaixa($nrorg, $cdfilial, $cdloja, $cdcaixa, $nmcaixa, $dtinclusao);
        $this->entityManager->persist($caixa);
        
        $clienfilial = $factoryCliente->criaClienfilial($nrorg, $cdfilial, $cdcliente, $dtinclusao);
        $this->entityManager->persist($clienfilial);

        $cdparavend = '1';
        $paravend = $factoryLoja->criaParavend($nrorg, $cdfilial, $cdparavend, $cdcliente, $dtinclusao);
        $this->entityManager->persist($paravend);
    }

    private function preencheTabelaOrganizacaooper(GeralEntities\Organizacao $entidadeOrganizacao, $cdoperador){
        $nrorganizacaooper = Utilities::generateNewCode($entidadeOrganizacao->getNrorg(), "ORGANIZACAOOPER", 1, 10, true, false, $this->entityManager->getConnection());
        $nrorg = $entidadeOrganizacao->getNrorg();
        $nrorgrel = $entidadeOrganizacao->getNrorg();
        $idativo = 'S';
        $dtinclusao = new SetformsDateTime();
        $cdoperinclusao = $this->environment->getCurrentUserId();
        $nrorginclusao = $this->environment->getCurrentOrganizationId();

        $entidadeOrganizacaoOperador = $this->factoryOrganizacaoOperador->preparaParaOrganizacao(
            $nrorganizacaooper, $nrorg, $cdoperador, $nrorgrel, $idativo, $dtinclusao,
            $cdoperinclusao, $nrorginclusao
        );

        $this->entityManager->persist($entidadeOrganizacaoOperador);
        return $nrorganizacaooper;
    }

    private function preencheTabelaEstruturaoper(GeralEntities\Organizacao $entidadeOrganizacao, $nrorganizacaooper, $nrestruturamrel){
        $nrorg = $entidadeOrganizacao->getNrorg();
        $nrestruturaoper = Utilities::generateNewCode($entidadeOrganizacao->getNrorg(), "ESTRUTURAOPER", 1, 10, true, false, $this->entityManager->getConnection());
        $dtinclusao = new SetformsDateTime();

        $entidadeEstruturaoper = $this->factoryEstruturaoper->cria(
            $nrorg, $nrestruturaoper, $nrorganizacaooper,
            $nrestruturamrel, $dtinclusao
        );

        $this->entityManager->persist($entidadeEstruturaoper);
    }

    public function defineStatusPreparado(EntityManager $defaultEntityManager, GeralEntities\Organizacao $entidade){
        $entidade = $this->factoryOrganizacao->defineStatusPreparado($entidade);
        $defaultEntityManager->persist($entidade);
    }

    private function insertCustomerPortalOperator(EntityManager $defaultEntityManager, $nrOrg, $entidade, $cdsenhaoperweb, $nmoperador, $dsemailoper) {
        if(!empty($entidade->getNrparcnegocio())) {
            $operadorRepository = InstanceProvider::getRepository()->getOperador();
            $clienteRepository  = InstanceProvider::getRepository()->getCliente();

            $cliente  = $clienteRepository->findOneBy(array('nrparcnegocio' => $entidade->getNrparcnegocio()));
            $operator = $operadorRepository->findOneBy(array('dsemailoper'  => $entidade->getDsemailoper(),
                                                             'cdcliente'    => $cliente->getCdcliente()));

            if (empty($operator)) {
                $currentNrorg      = $this->environment->getCurrentOrganizationId();
                $defaultConnection = $defaultEntityManager->getConnection();
                $cdOperador        = Utilities::generateNewCode($currentNrorg, "OPERADOR",     1, 12, false, false, $defaultConnection, true);
                $nrParcNegocio     = Utilities::generateNewCode($currentNrorg, "PARCNEGOCIO",  1, 10, true,  false, $defaultConnection, false);

                /** @var GeralEntities\Organizacao $organizacao */
                $organizacao = $defaultEntityManager->getRepository(Utilities::GERAL_ENTITY_DIR . 'Organizacao')->findOneBy(array('nrorg' => $nrOrg));
                /** @var GeralEntities\Cliente $cliente */
                $cliente = $defaultEntityManager->getRepository(Utilities::GERAL_ENTITY_DIR . 'Cliente')->findOneBy(array('nrorg' => $currentNrorg,
                    'nrparcnegocio' => $organizacao->getNrparcnegocio()));

                $dtinclusao     = new SetformsDateTime();
                $cdoperinclusao = $this->environment->getCurrentUserId();
                $nrorginclusao  = $this->environment->getCurrentOrganizationId();

                $parceiroNegocioCP = $this->factoryParceiroNegocio->criaParceiroNegocioCP($currentNrorg, $nrParcNegocio, $organizacao->getNmrazsocialorg(),
                    $organizacao->getNmorganizacao(), $dtinclusao, $cdoperinclusao, $nrorginclusao);
                $defaultEntityManager->persist($parceiroNegocioCP);
                $defaultEntityManager->flush();

                $nmoperadorEx = explode(' ', $nmoperador);
                $nmacesso     = strtolower(preg_replace("/[^A-Za-z]/", "", $nmoperadorEx[0] . end($nmoperadorEx)));

                $operadorCP   = $this->factoryOperador->criaOperadorCP($currentNrorg, $cliente->getCdcliente(),$cdsenhaoperweb,
                    $cdOperador, $nmoperador, $dsemailoper, $dtinclusao, $cdoperinclusao, $nrorginclusao, $nrParcNegocio,'N',
                    'S', $nmacesso);
                $defaultEntityManager->persist($operadorCP);
                $defaultEntityManager->flush();

                $this->preencheTabelaPerfacessparc($entidade, $nrParcNegocio, null, $defaultEntityManager);
                $defaultEntityManager->flush();

                $entidadeOperClie = $this->preencheTabelaOperClie($cdOperador, $cliente->getCdcliente());
                $defaultEntityManager->persist($entidadeOperClie);
            }
        }
    }

    private function enviaEmailComSenhaGerada(GeralEntities\Organizacao $entidade) {
        $cdoperador  = $this->codigoOperadorCriado;
        $dsemailoper = $this->emailOperadorCriado;
        $nmoperador  = $this->nomeOperadorCriado;
        $senhaDescriptografada = $this->senhaGeradaParaOperador;
        $this->serviceOperador->enviaSenhaPorEmail($cdoperador, $nmoperador, $dsemailoper, $senhaDescriptografada);
    }

    public function solicitaPreparacao($idOrganizacao, $email) {
        /** @var GeralEntities\Organizacao $entidadeOrganizacao */
        $entidadeOrganizacao = $this->repositoryOrganizacao->findOneBy(array('nrorg' => $idOrganizacao));
        $entidadeOrganizacao = $this->factoryOrganizacao->defineStatusSolicitado($entidadeOrganizacao);
        $this->entityManager->flush($entidadeOrganizacao);

        $nomeRecursoSolicitando = $this->repositoryOperador->findOneBy(array('cdoperador' => $this->environment->getCurrentUserId()))->getNmoperador();
        $nomeOrganizacao = $entidadeOrganizacao->getNmorganizacao();
        $idOrganizacao = $entidadeOrganizacao->getNrorg();
        $this->serviceEmailSolicitacao->enviaEmail($email, $nomeRecursoSolicitando, $idOrganizacao, $nomeOrganizacao);
    }

    public function changeEnvironment($idOrganizacao, $idOrgSaas) {
        /** @var GeralEntities\Organizacao $entidadeOrganizacao */
        $entidadeOrganizacao = $this->repositoryOrganizacao->findOneBy(array('nrorg' => $idOrganizacao));

        if($idOrgSaas == GeralEntities\Organizacao::IDORG_SAAS) {
            $prepStatus = GeralEntities\Organizacao::GERAR_ESTRUTURA;
        } else {
            $prepStatus = GeralEntities\Organizacao::PREPARACAO_PENDENTE_LOCAL_HOSTING;
        }

        $entidadeOrganizacao = $this->factoryOrganizacao->defineNovoStatus($entidadeOrganizacao, $prepStatus);
        $entidadeOrganizacao->setIdorgsaas($idOrgSaas);
        $this->entityManager->persist($entidadeOrganizacao);
        $this->entityManager->flush($entidadeOrganizacao);
    }

    public function changeStatusBkp($idOrganizacao, $prepStatusBkp) {
        /** @var GeralEntities\Organizacao $entidadeOrganizacao */
        $entidadeOrganizacao = $this->repositoryOrganizacao->findOneBy(array('nrorg' => $idOrganizacao));
        $this->factoryOrganizacao->defineNovoStatusBkp($entidadeOrganizacao, $prepStatusBkp);
    }

    public function setStatusInactive($idOrganizacao) {
        /** @var GeralEntities\Organizacao $entidadeOrganizacao */
        $entidadeOrganizacao = $this->repositoryOrganizacao->findOneBy(array('nrorg' => $idOrganizacao));
        $entidadeOrganizacao = $this->factoryOrganizacao->defineStatusInativo($entidadeOrganizacao);
        $this->entityManager->persist($entidadeOrganizacao);
        $this->entityManager->flush();
    }

    public function setStatusBlocked($idOrganizacao) {
        /** @var GeralEntities\Organizacao $entidadeOrganizacao */
        $entidadeOrganizacao = $this->repositoryOrganizacao->findOneBy(array('nrorg' => $idOrganizacao));
        $entidadeOrganizacao = $this->factoryOrganizacao->defineStatusBloqueado($entidadeOrganizacao);
        $this->entityManager->persist($entidadeOrganizacao);
        $this->entityManager->flush();
    }

    public function setStatusBackup($idOrganizacao) {
        /** @var GeralEntities\Organizacao $entidadeOrganizacao */
        $entidadeOrganizacao = $this->repositoryOrganizacao->findOneBy(array('nrorg' => $idOrganizacao));
        $entidadeOrganizacao = $this->factoryOrganizacao->defineStatusBackup($entidadeOrganizacao);
        $this->entityManager->persist($entidadeOrganizacao);
        $this->entityManager->flush();
    }

    public function setStatusStructureRemoved($idOrganizacao) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();

        try {
            /** @var GeralEntities\Organizacao $entidadeOrganizacao */
            $entidadeOrganizacao = $this->repositoryOrganizacao->findOneBy(array('nrorg' => $idOrganizacao));
            $entidadeOrganizacao = $this->factoryOrganizacao->defineStatusBaseRemovida($entidadeOrganizacao);
            $this->entityManager->persist($entidadeOrganizacao);

            if($entidadeOrganizacao->isSaas()) {
                $this->removeAllProductAccess($idOrganizacao);
            }

            $this->entityManager->flush();
            $connection->commit();
        } catch(\Exception $e) {
            if($connection->isTransactionActive()) {
                $connection->rollBack();
            }
            throw $e;
        }

    }

    public function setStatusPrepared($idOrganizacao) {
        /** @var GeralEntities\Organizacao $entidadeOrganizacao */
        $entidadeOrganizacao = $this->repositoryOrganizacao->findOneBy(array('nrorg' => $idOrganizacao));
        $entidadeOrganizacao = $this->factoryOrganizacao->defineStatusPreparado($entidadeOrganizacao);
        $this->entityManager->persist($entidadeOrganizacao);
        $this->entityManager->flush();
    }

    public function setStatusGenerateStructure($idOrganizacao) {
        /** @var GeralEntities\Organizacao $entidadeOrganizacao */
        $entidadeOrganizacao = $this->repositoryOrganizacao->findOneBy(array('nrorg' => $idOrganizacao));
        $entidadeOrganizacao = $this->factoryOrganizacao->defineStatusGerarEstrutura($entidadeOrganizacao);
        $this->entityManager->persist($entidadeOrganizacao);
        $this->entityManager->flush();
    }

    public function setTableTipoendereco(GeralEntities\Organizacao $entidadeOrganizacao)
    {
        $nrorg          = $entidadeOrganizacao->isSecundaria() ? $entidadeOrganizacao->getNrorgprincipal() : $entidadeOrganizacao->getNrorg();
        $dtinclusao     = new SetformsDateTime();
        $cdoperinclusao = $this->environment->getCurrentUserId();
        $nrorginclusao  = $this->environment->getCurrentOrganizationId();
        $tipoendereco   = $this->entityManager->getRepository(Utilities::GERAL_ENTITY_DIR . 'Tipoendereco')->findOneBy(array('cdtipoendereco' => 'PRINCIPAL'));
        if(empty($tipoendereco)) {
            $tipoendereco = $this->factoryEnderecoParc->createTipoendereco($nrorg, 'PRINCIPAL', 'Principal', 'S',
                $dtinclusao, $cdoperinclusao, $nrorginclusao);
            $this->entityManager->persist($tipoendereco);
            $this->entityManager->flush();
        }
    }

    public function setStatusPreparedZmart($idOrganizacao){
        $defaultEntityManager = $this->entityManager;
        $nrorgZmart = Utilities::getZmartNrorgParameter();
        $this->entityManager = $this->dbConnection->getOrgEntityManager($nrorgZmart);
        $conexao = $this->entityManager->getConnection();
        $conexaoDefault = $defaultEntityManager->getConnection();

        try {
            $conexao->beginTransaction();
            $conexaoDefault->beginTransaction();

            /** @var GeralEntities\Organizacao $entidadeOrganizacao */
            $entidadeOrganizacao = $this->repositoryOrganizacao->findOneBy(array('nrorg' => $idOrganizacao));
            $entidadeOrganizacao = $this->factoryOrganizacao->defineStatusPreparadoZmart($entidadeOrganizacao);
            $defaultEntityManager->persist($entidadeOrganizacao);

            if (!$entidadeOrganizacao->isSaas() && !$entidadeOrganizacao->isSecundaria()) {
                /** @var GeralEntities\Operador $operadorTeknisa */
                $operadorTeknisa = $this->repositoryOperador->findOneBy(array('dsemailoper' => $entidadeOrganizacao->getDsemailoper()));
                if(empty($operadorTeknisa)) {
                    throw new \Exception(Messages::ERR_ORGANIZATION_OPERATOR_NOT_FOUND);
                }
                $nrperfilacesso = $this->preencheTabelaPerfilacesso($entidadeOrganizacao, true);
                $this->setTableTipoendereco($entidadeOrganizacao);
//                $this->createClientZmart($entidadeOrganizacao, $nrorgZmart);
                $this->insertZmartOperator($entidadeOrganizacao, $nrperfilacesso, $nrorgZmart, $operadorTeknisa);
                $this->entityManager->flush();
                $conexao->commit();
            }

            $defaultEntityManager->flush();
            $conexaoDefault->commit();
        } catch(\Exception $ex){
            if($conexao->isTransactionActive()) {
                $conexao->rollback();
            }
            if($conexaoDefault->isTransactionActive()) {
                $conexaoDefault->rollback();
            }
            throw $ex;
        }
    }

    public function createClientZmart(GeralEntities\Organizacao $organizacao, $nrorgZmart) {
        $currentNrorg   = $organizacao->isSecundaria() ? $organizacao->getNrorgprincipal() : $organizacao->getNrorg();
        $nrEnderecoParc = $organizacao->getNrparcnegocio();
        $nrParcNegocio  = $this->preencheTabelaParcnegocio($organizacao, self::TIPO_CLIENTE_PARCEIRO, $currentNrorg, $nrorgZmart);
        $this->entityManager->flush();

        $this->preencheTabelaEnderecoParc($organizacao, $nrParcNegocio, $nrEnderecoParc, $currentNrorg);

        $nrinsjurclie = Utilities::removeCaracteresEspeciais($organizacao->getNrinsjurempr());
        $cdcliente    = $organizacao->getIdtipoinscrorg() . $nrinsjurclie;
        $cliente      = $this->entityManager->getRepository(Utilities::GERAL_ENTITY_DIR . 'Cliente')->findOneBy(array('cdcliente' => $cdcliente));
        if(empty($cliente)) {
            $cdcliente = $this->preencheTabelaCliente($organizacao, $nrParcNegocio, $currentNrorg);
        }
        $this->entityManager->flush();

        $this->preencheTabelaEndeClie($organizacao, $cdcliente, $nrEnderecoParc, $currentNrorg);
        $this->entityManager->flush();
    }

    private function insertZmartOperator(GeralEntities\Organizacao $organizacao, $nrperfilacesso, $nrorgZmart, GeralEntities\Operador $operadorTeknisa)
    {
        $nrorg        = $organizacao->isSecundaria() ? $organizacao->getNrorgprincipal() : $organizacao->getNrorg();
        $currentNrorg = $nrorgZmart ? $nrorgZmart : $nrorg;

        $connection    = $this->entityManager->getConnection();
        $cdOperador    = Utilities::generateNewCode($currentNrorg, "OPERADOR",    1, 12, false,false, $connection, false);
        $nrParcNegocio = Utilities::generateNewCode($currentNrorg, "PARCNEGOCIO", 1, 10, true, false, $connection, false);

        $dtinclusao     = new SetformsDateTime();
        $cdoperinclusao = $this->environment->getCurrentUserId();
        $nrorginclusao  = $this->environment->getCurrentOrganizationId();

        $parceiroNegocioCP = $this->factoryParceiroNegocio->criaParceiroNegocioCP($nrorg, $nrParcNegocio, $organizacao->getNmrazsocialorg(),
            $organizacao->getNmorganizacao(), $dtinclusao, $cdoperinclusao, $nrorginclusao);
        $this->entityManager->persist($parceiroNegocioCP);
        $this->entityManager->flush();

        $cdsenhaoperweb = $operadorTeknisa->getCdsenhaoperweb();
        $nmoperador     = $operadorTeknisa->getNmoperador();
        $dsemailoper    = $operadorTeknisa->getDsemailoper();
        $nmacesso       = $operadorTeknisa->getNmacesso();
        $operadorCP     = $this->factoryOperador->criaOperadorCP($nrorg, null, $cdsenhaoperweb, $cdOperador,
            $nmoperador, $dsemailoper, $dtinclusao, $cdoperinclusao, $nrorginclusao, $nrParcNegocio,'N', 'S', $nmacesso);
        if($organizacao->isSecundaria()) {
            $operadorCP->setNrorgoper($organizacao->getNrorg());
        }

        $this->entityManager->persist($operadorCP);
        $this->entityManager->flush();
        $this->preencheTabelaPerfacessparc($organizacao, $nrParcNegocio, $nrperfilacesso, NULL, $nrorgZmart);
        $this->entityManager->flush();
    }

    public function requestDeactivation($idOrganizacao, $email){
        $entidadeOrganizacao = $this->repositoryOrganizacao->findOneBy(array('nrorg' => $idOrganizacao));
        $nomeRecursoSolicitando = $this->repositoryOperador->findOneBy(array('cdoperador' => $this->environment->getCurrentUserId()))->getNmoperador();
        $nomeOrganizacao = $entidadeOrganizacao->getNmorganizacao();
        $idOrganizacao = $entidadeOrganizacao->getNrorg();
        $this->serviceEmailSolicitacao->sendEmailDeactivation($email, $nomeRecursoSolicitando, $idOrganizacao, $nomeOrganizacao);
    }

    public function habilitaPreparacao($id){
        /** @var GeralEntities\Organizacao $entidadeOrganizacao */
        $entidadeOrganizacao = $this->repositoryOrganizacao->findOneBy(array('nrorg' => $id));
        $entidadeOrganizacao = $this->factoryOrganizacao->defineStatusProntoParaPreparacao($entidadeOrganizacao);
        $this->entityManager->flush($entidadeOrganizacao);
    }

    public function execStructureCommand($command, $commandType, $url, $environment, $orgId) {
        ini_set('max_execution_time', 3600);

        $isNotSaasOrg     = $environment != self::IS_ORGANIZATION_SAAS;
        $auxEntityManager = $this->dbConnection->getOrgEntityManager($orgId, true, $isNotSaasOrg);

        $url   .= Utilities::getBaseUriParameter();
        $params = array(
            "requestType" => "Row",
            "row" => array (
                "COMMAND"      => $command,
                "COMMAND_TYPE" => $commandType,
                "CONNECTION"   => $auxEntityManager->getConnection()->getParams()
            )
        );

        $result = $this->utilities->chamaWebService(null, '/executeCommand', $params, 'result', Utilities::SECRET_CLIENTE_LIBRARIES, $url, 3600);
        if(empty($result)) {
            throw new \Exception(Messages::ERR_EMPTY_RESULT_CALL_WEBSERVICE);
        }
        return $result;
    }

    public function getStructureCommands($environment, $action) {
        $orgId    = InstanceProvider::getEnvironment()->getCurrentOrganizationId();
        $filter   = " AND IDAMBCOMANDO = '$environment' AND IDACAOEXECOM = '$action' AND IDATIVO = 'S' ";
        $commands = InstanceProvider::getRepository()->getOmComandoEstrutura()->findCommands($orgId, $filter);

        if(empty($commands)) {
            throw new \Exception(Messages::ERR_NOT_COMMANDS_ENVIRONMENT_AND_ACTION);
        }

        $commandsWithoutUrls = [];
        foreach ($commands as &$command) {
            $filterUrl = " AND IDATIVO = 'S' ";
            $command['URLS'] = InstanceProvider::getRepository()->getOmComandoEstrutura()->findCommandUrls($orgId, $command['ID'], $filterUrl);
            if(empty($command['URLS'])) {
                $commandsWithoutUrls[] = '"' . $command['ID'] . ' - ' . $command['DSCOMANDOEST'] . '"';
            }
        }

        if(!empty($commandsWithoutUrls)) {
            $commandsError = join('<br/>', $commandsWithoutUrls);
            throw new \Exception(str_replace('{{commands}}', $commandsError, Messages::ERR_COMMANDS_WITHOUT_URLS));
        }

        return $commands;
    }

    public function getOrganizationRestrictionByIp($orgId, $filters){
        return $this->repositoryParamOrg->getOrganizationRestrictionByIp($orgId, Utilities::createQueryFiltered($filters), Utilities::getOrgDbUserName($orgId));
    }

    public function isIpValid($ip, $ipVersion){
        if ($ipVersion == "IPV4"){
            return filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_IPV4) ? true : false;
        } else if ($ipVersion == "IPV6") {
            return filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_IPV6) ? true : false;
        } else {
            return false;
        }
    }

    public function removeOrganizationRestrictionByIp($restrictionId, $orgId) {
        $this->entityManager = $this->dbConnection->getOrgEntityManager($orgId);
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();
            if ($restrictionId){
                $this->repositoryIpRestriction->setEntityManager($this->entityManager);
                $organizationRestriction = $this->repositoryIpRestriction->findOneBy(array('nrIpRestriction' => $restrictionId, 'nrorg' => $orgId));
                if ($organizationRestriction){
                    $this->entityManager->remove($organizationRestriction);
                    $this->entityManager->flush();
                    $this->entityManager->commit();
                }
            }
        } catch(\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * @param $ipRestrictedId
     * @param $ipv4
     * @param $ipv6
     * @param $orgId
     * @param $description
     * @param $active
     * @return array
     * @throws \Exception
     */
    public function saveOrganizationRestrictionByIp($ipRestrictedId, $ipv4, $ipv6, $orgId, $description, $active){
        $this->entityManager = $this->dbConnection->getOrgEntityManager($orgId);
        $connection = $this->entityManager->getConnection();
        try {
            $dateNow = new \DateTime();
            $codOperador = $this->environment->getCurrentUserId();
            $currentOrgId = $this->environment->getCurrentOrganizationId();
            $connection->beginTransaction();
            $this->validateIps($ipv4, $ipv6);
            $ipv4 ? $ipv6 = null : $ipv4 = null;

            if ($ipRestrictedId){
                $savedRestriction = $this->updateRestriction($ipRestrictedId, $ipv4, $ipv6, $description, $active, $orgId,
                    $currentOrgId, $codOperador, $dateNow);
            } else {
                $savedRestriction = $this->createRestrictionIp($ipv4, $ipv6, $description, $active, $orgId, $currentOrgId, $codOperador, $dateNow);
            }

            if (!$savedRestriction){
                throw new \Exception(Messages::ERR_NOT_SAVED);
            }

            $this->entityManager->persist($savedRestriction);
            $this->entityManager->flush();
            $connection->commit();
            return array("ID" => $savedRestriction->getNrIpRestriction());
        } catch(\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    private function validateIps($ipv4, $ipv6){
        if (!$ipv4 && !$ipv6) {
            throw new \Exception(Messages::ERR_NONE_IP_INFORMED);
        } else {
            if ($ipv4 && !$this->isIpValid($ipv4, "IPV4")) {
                throw new \Exception(Messages::ERR_IP_INVALID);
            } else if ($ipv6 && !$this->isIpValid($ipv6, "IPV6")) {
                throw new \Exception(Messages::ERR_IP_INVALID);
            }
        }
    }

    private function updateRestriction($ipRestrictedId, $ipv4, $ipv6, $description, $active, $orgId, $currentOrgId, $codOperador, $dateNow){
        $this->repositoryIpRestriction->setEntityManager($this->entityManager);
        $ipRestricted = $this->repositoryIpRestriction->findOneBy(array('nrIpRestriction' => $ipRestrictedId, 'nrorg' => $orgId));
        if ($ipRestricted) {
            if ($ipRestricted->getDsIpv4Restricted() == $ipv4 && $ipRestricted->getDsIpv6Restricted() == $ipv6){
                return $this->factoryIpRestriction->save($ipRestricted, $orgId, $currentOrgId, $codOperador, $dateNow,
                    $description, $ipv4, $ipv6, $active, $this->entityManager);
            } else {
                $this->checkIfIpAlreadyRegistered($orgId, $ipv4, $ipv6);
                return $this->factoryIpRestriction->save($ipRestricted, $orgId, $currentOrgId, $codOperador, $dateNow,
                    $description, $ipv4, $ipv6, $active, $this->entityManager);
            }
        } else {
            throw new \Exception(Messages::ERR_RESTRICTION_CHANGES_NOT_SAVED);
        }
    }

    private function checkIfIpAlreadyRegistered($orgId, $ipv4, $ipv6){
        $this->repositoryIpRestriction->setEntityManager($this->entityManager);
        if ($ipv4){
            if ($this->repositoryIpRestriction->findOneBy(array('nrorg' => $orgId, 'dsIpv4Restricted' => $ipv4))) {
                throw new \Exception(Messages::ERR_IP_ALREADY_REGISTERED);
            }
        } else {
            if ($this->repositoryIpRestriction->findOneBy(array('nrorg' => $orgId, 'dsIpv6Restricted' => $ipv6))) {
                throw new \Exception(Messages::ERR_IP_ALREADY_REGISTERED);
            }
        }
    }

    private function createRestrictionIp($ipv4, $ipv6, $description, $active, $orgId, $currentOrgId, $codOperador, $dateNow) {
        $this->checkIfIpAlreadyRegistered($orgId, $ipv4, $ipv6);
        return $this->factoryIpRestriction->save(null, $orgId, $currentOrgId, $codOperador, $dateNow, $description, $ipv4, $ipv6, $active, $this->entityManager);
    }

    public function getOrganizationParameters($filters) {
        $orgId = Utilities::getValueInFilterDataSourceByName($filters, 'NRORG');
        $parameters = $this->repositoryParamOrg->getOrganizationParameters($orgId);
        $organization = InstanceProvider::getRepository()->getOrganizacao()->findOneBy(array('nrorg' => $orgId));
        if($organization->getNrorgprincipal()) {
            $mainParameters = $this->repositoryParamOrg->getOrganizationParameters($organization->getNrorgprincipal());
        } else {
            $mainParameters = null;
        }

        $nrparcnegocio = $organization->getNrparcnegocio();
        $dbUserValidation = $this->repositoryParamOrg->getOrganizationDbUser(Utilities::getOrgDbUserName($orgId));
        $notDbUser = empty($dbUserValidation);

        $financialDefault = $this->repositoryParamOrg->getOrganizationFinancialDefaultInfo($orgId);
        if($financialDefault) {
            $dtvalarqpers     = empty($financialDefault[0]['DTVALARQPERS']) && $financialDefault[0]['DTVALARQPERS'] != 0 ? null : $financialDefault[0]['DTVALARQPERS'];
            $nrdiavalpers     = empty($financialDefault[0]['NRDIAVALPERS']) && $financialDefault[0]['NRDIAVALPERS'] != 0 ? null : $financialDefault[0]['NRDIAVALPERS'];

        } else {
            $dtvalarqpers = $nrdiavalpers = null;
        }
        if(empty($parameters)) {
            $parameters[0] = array (
                'NOT_DB_USER' => $notDbUser,
                'ORG_ID'      => $orgId,
                'IDUTWALLET'  => 'N',
                'DTVALARQPERS' => $dtvalarqpers,
                'NRDIAVALPERS' => $nrdiavalpers

            );
        } else {
            $parameters[0]['NOT_DB_USER'] = $notDbUser;
            $parameters[0]['NRDIAVALPERS'] = $nrdiavalpers;
            $parameters[0]['DTVALARQPERS'] = $dtvalarqpers;
        }
        $parameters[0]['NRPARCNEGOCIO'] = $nrparcnegocio;

        if($organization->getNrorgprincipal()) {
            $parameters[0]['IDCOLETOR'] =            !empty($mainParameters) ? $mainParameters[0]['IDCOLETOR'] : null;
            $parameters[0]['DSSTRINGCONEXAO'] =      !empty($mainParameters) ? $mainParameters[0]['DSSTRINGCONEXAO'] : null;
            $parameters[0]['IDUTWALLET'] =           !empty($mainParameters) ? $mainParameters[0]['IDUTWALLET'] : null;
            $parameters[0]['DSUSUARIOBANCO'] =       !empty($mainParameters) ? $mainParameters[0]['DSUSUARIOBANCO'] : null;
            $parameters[0]['DSSENHABANCO'] =         !empty($mainParameters) ? $mainParameters[0]['DSSENHABANCO'] : null;
            $parameters[0]['IDTPBANCO'] =            !empty($mainParameters) ? $mainParameters[0]['IDTPBANCO'] : null;
            $parameters[0]['NMTPBANCO'] =            !empty($mainParameters) ? $mainParameters[0]['NMTPBANCO'] : null;
            $parameters[0]['USING_MAIN_COLLECTOR'] = 'S';
        } else {
            $parameters[0]['USING_MAIN_COLLECTOR'] = 'N';
        }

        return $parameters;
    }

    /**
     * @param $orgId
     * @param $useRestriction
     * @param $allowProductCall
     * @return array
     * @throws \Exception
     */
    public function saveOrganizationParameters($orgId, $useRestriction, $allowProductCall, $idColetor,
                                               $dsStringConexao, $idUtWallet, $dsUsuarioBanco, $dsSenhaBanco, 
                                               $idTpBanco, $dtvalarqpers, $nrdiavalpers, $idUtRedirectRc) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {

            $dateNow      = new \DateTime();
            $currentOrgId = $this->environment->getCurrentOrganizationId();
            $codOperador  = $this->environment->getCurrentUserId();

            /** @var TeknisaEntities\OmParamOrg $organizationParameters */
            $organizationParameters = $this->repositoryParamOrg->findOneBy(array('nrorg' => $orgId));
            if($organizationParameters) {
                if(!empty($dsSenhaBanco) && $organizationParameters->getDssenhabanco() != $dsSenhaBanco) {
                    $dsSenhaBanco = OpensslCrypt::encrypt($dsSenhaBanco);
                }
            } else {
                $dsSenhaBanco = OpensslCrypt::encrypt($dsSenhaBanco);
            }
            $savedParameters = $this->factoryOrganizationParameters->save($organizationParameters, $orgId,
                $currentOrgId, $codOperador, $dateNow, $useRestriction, $allowProductCall, $idColetor,
                $dsStringConexao, $idUtWallet, $dsUsuarioBanco, $dsSenhaBanco, $idTpBanco, $idUtRedirectRc);
            $this->entityManager->persist($savedParameters);
            $this->entityManager->flush();

            $notDbUser = true;
            $organization = $this->repositoryOrganizacao->findOneBy(array('nrorg' => $orgId));
            if($organization->getIdorgsaas() == 'S') {
                $dbUserValidation = $this->repositoryParamOrg->getOrganizationDbUser(Utilities::getOrgDbUserName($orgId));

                if(!empty($dbUserValidation)) {
                    $orgEntityManager = $this->dbConnection->getOrgEntityManager($orgId);
                    $orgConnection = $orgEntityManager->getConnection();
                    $orgConnection->beginTransaction();
                    try {
                        $this->repositoryParamOrg->setEntityManager($orgEntityManager);
                        /** @var TeknisaEntities\OmParamOrg $orgOrganizationParameters */
                        $orgOrganizationParameters = $this->repositoryParamOrg->findOneBy(array('nrorg' => $orgId));
                        $orgSavedParameters = $this->factoryOrganizationParameters->save($orgOrganizationParameters, $orgId,
                            $currentOrgId, $codOperador, $dateNow, $useRestriction, $allowProductCall, $idColetor,
                            $dsStringConexao, $idUtWallet, $dsUsuarioBanco, $dsSenhaBanco, $idTpBanco, $idUtRedirectRc);
                        $orgEntityManager->persist($orgSavedParameters);
                        $notDbUser = false;

                        $orgEntityManager->flush();
                        $orgEntityManager->commit();
                    } catch(\Exception $e) {
                        if($orgConnection->isTransactionActive()) {
                            $orgConnection->rollBack();
                        }
                    }
                }
            }
             $organization = InstanceProvider::getRepository()->getOrganizacao()->findOneBy(array('nrorg' => $orgId));
             $nrparcnegocio = $organization->getNrparcnegocio();
             if($nrparcnegocio){
                $cliente    = $this->repositoryCliente->findOneBy(array('nrparcnegocio' => $nrparcnegocio));
                if($dtvalarqpers){
                    $date = \DateTime::createFromFormat('d/m/Y', $dtvalarqpers);
                    $cliente->setDtvalarqpers($date);

                }else{
                    $cliente->setDtvalarqpers(null);
                }
                $cliente->setNrdiavalpers($nrdiavalpers);
                $this->entityManager->persist($cliente);
            }

            $this->entityManager->flush();
            $connection->commit();
            return array('NOT_DB_USER' => $notDbUser);
        } catch(\Exception $e) {
            if($connection->isTransactionActive()) {
                $connection->rollBack();
            }
            throw $e;
        }
    }

    public function getDefaultParameters($filters) {
        $conditions = Utilities::createQueryFiltered($filters, 'OPD');
        return InstanceProvider::getRepository()->getOmParamOrgDefault()->getDefaultParameters($conditions);
    }

    public function saveDefaultParameters($id, $idColetor,  $dsStringConexao, $idUtWallet, $dsUsuarioBanco,
                                          $dsSenhaBanco, $idativo, $idamborg, $idTpBanco) {
        $dateNow      = new \DateTime();
        $currentOrgId = $this->environment->getCurrentOrganizationId();
        $codOperador  = $this->environment->getCurrentUserId();

        /** @var TeknisaEntities\OmParamOrgDefault $sameTypeRegister */
        $sameTypeRegister = InstanceProvider::getRepository()->getOmParamOrgDefault()->findOneBy(array('idamborg' => $idamborg));
        if($sameTypeRegister && $sameTypeRegister->getId() != $id) {
            throw new \Exception(Messages::ERR_EXISTS_OTHER_DEFAULT_PARAMETER_WITH_TYPE);
        }

        /** @var TeknisaEntities\OmParamOrgDefault $defaultParameters */
        $defaultParameters = $id ? InstanceProvider::getRepository()->getOmParamOrgDefault()->find($id) : null;
        if($defaultParameters) {
            if(!empty($dsSenhaBanco) && $defaultParameters->getDssenhabanco() != $dsSenhaBanco) {
                $dsSenhaBanco = OpensslCrypt::encrypt($dsSenhaBanco);
            }
        } else {
            $dsSenhaBanco = OpensslCrypt::encrypt($dsSenhaBanco);
        }
        $savedParameters = $this->factoryOrganizationParameters->saveDefaultParams($defaultParameters,
            $currentOrgId, $codOperador, $dateNow, $idColetor, $dsStringConexao, $idUtWallet,
            $dsUsuarioBanco, $dsSenhaBanco, $idativo, $idamborg, $idTpBanco);
        $this->entityManager->persist($savedParameters);
        $this->entityManager->flush();
        return array(ltrim($savedParameters->getId(), '0'));
    }

    public function removeDefaultParameters($id) {
        $defaultParameters = InstanceProvider::getRepository()->getOmParamOrgDefault()->find($id);
        $this->entityManager->remove($defaultParameters);
        $this->entityManager->flush();
    }

    /**
     * @param $orgId
     * @return array
     * @throws \Exception
     */
    public function generateLoginAccessFile($orgId) {
        $orgProducts = $this->repositoryOmProdutoOrg->findBy(array('nrorgId' => $orgId));

        if(empty($orgProducts)) {
            throw new \Exception(Messages::ERR_NOT_ASSOCIATED_ORGANIZATION_PRODUCTS);
        } else {
            $accessFile = array('products' => array());
            $productsWithoutElements = array();

            /** @var TeknisaEntities\OmProdutoOrg $orgProduct */
            foreach ($orgProducts as $orgProduct) {
                /** @var TeknisaEntities\OmProduto $omProduct */
                $omProduct  = $this->repositoryOmProduto->findOneBy(array('id' => $orgProduct->getNrprodutoId()));
                $elementIds = $this->getProductElementIds($orgProduct->getId());

                if(empty($elementIds)) {
                    $productsWithoutElements[] = $omProduct->getNmproduto();
                } else {
                    $accessFile['products'][] = array(
                        'productId' => $omProduct->getNrexternalProductId(),
                        'elements'  => $elementIds
                    );
                }
            }
        }

        $this->createLoginAccessFile($orgId, $accessFile);
        return $productsWithoutElements;
    }

    /**
     * @param $orgProductId
     * @return array
     */
    private function getProductElementIds($orgProductId) {
        $orgElements = $this->repositoryOmProdutoOrgElemento->findBy(array('nrprodutoOrgId' => $orgProductId));
        $elementIds  = array();
        /** @var TeknisaEntities\OmProdutoOrgElemento $orgElement */
        foreach($orgElements as $orgElement) {
            /** @var TeknisaEntities\OmProdutoElemento $element */
            $element = $this->repositoryOmProdutoElemento->findOneBy(array('id' => $orgElement->getNrprodutoElementoId()));
            if(!empty($element)) {
                $elementIds[] = $element->getUniqueId();
            }
        }

        return $elementIds;
    }

    /**
     * @param $orgId
     * @param $accessFile
     */
    private function createLoginAccessFile($orgId, $accessFile) {
        $encryptedAccess = Utilities::zeedhiEncrypt(json_encode($accessFile));
        list($orgFileName, $orgFileDir) = $this->getOrgAccessFileInfo($orgId);
        file_put_contents($orgFileDir, $encryptedAccess);
    }

    /**
     * @param $orgId
     */
    public function downloadLoginAccessFile($orgId) {
        list($orgFileName, $orgFileDir) = $this->getOrgAccessFileInfo($orgId);
        Utilities::downloadAndRemoveFile($orgFileDir, $orgFileName);
    }

    /**
     * @param $orgId
     */
    public function downloadStructureExportation($orgId) {
        $fileName = $this->utilities->getOrgDbUserName($orgId) . '.zip';
        $file     = $this->utilities->getExportStructurePath() . '/' . $fileName;
        Utilities::downloadAndRemoveFile($file, $fileName, 'application/zip', false);
    }

    /**
     * @param $orgId
     * @return array
     */
    private function getOrgAccessFileInfo($orgId) {
        $orgFileName = 'oAcc' . $orgId . '.tek';
        $orgFileDir = $this->utilities->getLogFileDir($orgFileName);
        return array($orgFileName, $orgFileDir);
    }

    /**
     * @param $nrIncricaoOrg
     * @param $tipoOrg
     * @return int|null
     */
    public function saveOrganizationValidation($nrIncricaoOrg, $tipoOrg) {
        $nrIncricaoOrg = Utilities::removeCaracteresEspeciais($nrIncricaoOrg);
        /** @var GeralEntities\Cliente $cliente */
        $cliente = $this->repositoryCliente->findOneBy(array('nrinsjurclie' => $nrIncricaoOrg, 'idtpijurclie' => $tipoOrg));
        $validations = array();
        if(is_object($cliente)) {
            $validations['NRPARCNEGOCIO'] = $cliente->getNrparcnegocio();
            $validations['NMFANTCLIE']    = $cliente->getNmfantclie();
        }

        return $validations;
    }

    public function createClient(GeralEntities\Organizacao $organizacao) {
        $currentNrorg   = $this->environment->getCurrentOrganizationId();
        $nrEnderecoParc = Utilities::generateNewCode($currentNrorg, "ENDERECOPARC", 1, 10, true, false, $this->entityManager->getConnection());

        $nrParcNegocio = $this->preencheTabelaParcnegocio($organizacao, self::TIPO_CLIENTE_PARCEIRO, $currentNrorg);
        $this->entityManager->flush();

        $this->preencheTabelaEnderecoParc($organizacao, $nrParcNegocio, $nrEnderecoParc, $currentNrorg);
        $cdcliente = $this->preencheTabelaCliente($organizacao, $nrParcNegocio, $currentNrorg);
        $this->entityManager->flush();

        $this->preencheTabelaParceria($organizacao, $nrParcNegocio, self::TIPO_CLIENTE_PARCEIRO, $currentNrorg);
        $this->preencheTabelaEndeClie($organizacao, $cdcliente, $nrEnderecoParc, $currentNrorg);

        $organizacao->setNrparcnegocio($nrParcNegocio);
        $this->entityManager->persist($organizacao);
        $this->entityManager->flush();
    }

    public function saveOrganizationAppCode($dschavenotif, $cdappnotif, $productOrgId, $idProdSaas, $urlRedirect, $idProdutoExclusivo, 
        $urlAcesso, $dtvalarqpers, $idUtRedirectRc, $idProdutoAssistant, $idUtAssMultVer, $urlCustomization)
    {
        $connection = $this->entityManager->getConnection();
        $productOrgRepository = InstanceProvider::getRepository()->getOmProdutoOrg();
        $organizacaoRepository = InstanceProvider::getRepository()->getOrganizacao();
        $productOrgEntity = null;
        try {
            $connection->beginTransaction();
            /** @var TeknisaEntities\OmProdutoOrg $productOrgEntity */
            $productOrgEntity = $productOrgRepository->findOneBy(array('id' => $productOrgId));
            $productOrgEntity->setCdappnotif($cdappnotif);
            $productOrgEntity->setDschavenotif($dschavenotif);
            $productOrgEntity->setDsurlredirect($urlRedirect);
            $productOrgEntity->setIdutredirectrc($idUtRedirectRc);
            $productOrgEntity->setIdprodutoassistant($idProdutoAssistant);
            $productOrgEntity->setIdutassmultver($idUtAssMultVer);
            $productOrgEntity->setIdprodutoexclusivo($idProdutoExclusivo);
            $productOrgEntity->setDsurlacesso($urlAcesso);
            $productOrgEntity->setDsurlcustom($urlCustomization);
            if($dtvalarqpers){
                $date = \DateTime::createFromFormat('d/m/Y', $dtvalarqpers);
                $productOrgEntity->setDtvalarqpers($date);
            }else{
                $productOrgEntity->setDtvalarqpers(null);
            }
            $idProdSaasOrg = $idProdSaas === OmProduto::HERDA_ORGANIZACAO ? $organizacaoRepository->findOneBy(array('nrorg' => $productOrgEntity->getNrorgId()))->getIdorgsaas() : $idProdSaas;
            $productOrgEntity->setIdprodutosaas($idProdSaasOrg);
            $this->entityManager->persist($productOrgEntity);
            $this->entityManager->flush();
            $connection->commit();
            return array('ID' => $productOrgEntity->getId());
        } catch (Exception $ex) {
            $connection->rollback();
            throw $ex;
        }
    }

    public function getProductOrg($conditions) {
        $productOrgRepository = InstanceProvider::getRepository()->getOmProdutoOrg();
        $id = Utilities::getValueInFilterDataSourceByName($conditions,"ID");
        $product_org = Utilities::getValueInFilterDataSourceByName($conditions,"PRODUCT_ORG");
        return $productOrgRepository->findProductOrg($id,$product_org);
    }

    public function saveOrganizationLog($log, $orgId) {
        InstanceProvider::getRepository()->getOrganization()->saveOrganizationLogDir($log, $orgId);
    }

    public function verifyBaseErrors($orgId){
        set_time_limit(900);
        $organizationRepository = InstanceProvider::getRepository()->getOrganization();
        return $organizationRepository->verifyBaseErrors($orgId);
    }

    public function getOrganizationLog($orgId) {
        $log    = '';
        $orgLog = InstanceProvider::getRepository()->getOrganization()->getOrganizationLogDir($orgId);
        if($orgLog['DSDIRLOG']) {
            return file_get_contents(Utilities::getFileServerFilesUrlParameter() . $orgLog['DSDIRLOG']);
        }
        return $log;
    }

    public function downloadOrganizationLog($orgId) {
        $orgLog = InstanceProvider::getRepository()->getOrganization()->getOrganizationLogDir($orgId);
        Utilities::downloadAndRemoveFile(Utilities::getFileServerFilesUrlParameter() . $orgLog['DSDIRLOG'], 'LOG_' . $orgId . '.txt',
                                        'text/plain', false);
    }

    public function validaGeracaoDadosInicias(GeralEntities\Organizacao $entidadeOrganizacao) {
        $repositoryPais = InstanceProvider::getRepository()->getPais();
        $repositoryPais->setEntityManager($this->entityManager);
        $pais = $repositoryPais->findOneBy(array('cdpais' => $entidadeOrganizacao->getCdpais()));
        if(empty($pais)) {
            throw new \Exception(Messages::ERR_NOT_PRE_INITIAL_DATA);
        }
    }

    public function removeAllProductAccess($orgId) {
        /** @var TeknisaEntities\OmProdutoOrg[] $orgProducts */
        $orgProducts = $this->repositoryOmProdutoOrg->findBy(array('nrorgId' => $orgId));
        foreach ($orgProducts as $orgProduct) {
            $orgElements = $this->repositoryOmProdutoOrgElemento->findBy(array('nrprodutoOrgId' => $orgProduct->getId()));
            /** @var TeknisaEntities\OmProdutoOrgElemento[] $orgElements */
            foreach($orgElements as $orgElement) {
                $this->entityManager->remove($orgElement);
            }
            $this->entityManager->flush();
            $this->entityManager->remove($orgProduct);
        }
    }
}
